<?php

/*
 * Holds a stack of activity and view invocations
 *
 */

 class ACF_InvocationStackAdvice {
    private $popStack;
    private $clearStack;

    function __construct() {
        $this->popStack = false;
        $this->pushStack = false;
        $this->clearStack = false;
    }

    function ShouldPushStack() {
        return $this->pushStack;
    }

    function ShouldPopStack() {
        return $this->popStack;
    }

    function ShouldClearStack() {
        return $this->clearStack;
    }

    function SetPushStack($flag) {
        return $this->pushStack = !!$flag;
    }

    function SetPopStack($flag) {
        return $this->popStack = !!$flag;
    }

    function SetClearStack($flag) {
        return $this->clearStack = !!$flag;
    }
}

class ACF_InvocationStack {
    private $stack;
    private $history;

    function __construct() {
        $this->History(false);
        $this->ClearStack();
    }

    function ProcessEventResponse(ACF_EventResponse $response) {
        $advice = $response->InvocationStackAdvice();
        if($advice->ShouldClearStack())
            $this->ClearStack();
        if($advice->ShouldPopStack())
            $this->PopStack();

        // Add new invocations to stack, if any
        foreach($response->InvocationsArray() as $invocation)
            $this->PushInvocation($invocation);
    }

    /*
     * Keep invocation stack history
     *
     * arg === false - disable history
     * arg === true - enable history
     * arg === anything else, add to history if history is enabled
     *
     */

    function History($arg) {
        static $timeformat = '%Y-%m-%d %H:%M:%S';

        if($arg === false) {
            $this->history = false;
            return;
        }

        list($usec, $sec) = explode(" ", microtime());
        $stamp = strftime($timeformat).'.'.sprintf('%06d',$usec);

        if($arg === true) {
            if(!$this->history)
                $this->history = array("$stamp\tStarted keeping history");
            return;
        }

        if($this->history)
            $this->history[] = "$stamp\t".$arg;

    }

    /*
     * Return history as a string
     *
     */

    function HistoryString() {
        if(!$this->history)
            return '';
        $output = '';
        foreach($this->history as $entry)
            $output .= $entry."\n";
//echo '((output:';print_r($output);echo "))\n";
        return $output;
    }

    /*
     * History as an array
     *
     */

     function HistoryArray() {
         return $this->history ? $this->history() : array();
     }

     function LogEventToHistory(ACF_EventSource $source, ACF_Event $event) {
         $msg = '** EVENT: '.$source->Realm().':'.$source->Name();
         $msg .= ' '.$event->Name();
         $this->History($msg);
     }

    function ClearStack() {
        $this->History('Cleared stack');
        $this->stack = array();
    }

    function PopStack() {

        if(empty($this->stack)) {
            $this->History('PopStack (was empty)');
            return NULL;
        }
        $invocation = array_pop($this->stack);
        $this->History('PopStack()=='.$invocation->TaskType() . ' ' .$invocation->TaskName() .  ' ('.count($this->stack).' items remaining)');
        return $invocation;
    }

    function TopOfStack() {
        if(empty($this->stack)) {
            $this->History('TopOfStack (is empty)');
            return NULL;
        }
        $top = clone $this->stack[count($this->stack)-1];
        $this->history('TopOfStack returning '.$top->TaskType() . ' ' .$top->TaskName());
        return $top;
    }

    function PushInvocation(ACF_Invocation $invocation) {
        $this->stack[] = $invocation;
        $this->history('PushInvocation('.$invocation->TaskType() . ' ' .$invocation->TaskName().'), (now '.(count($this->stack)).' items)');
    }

    function StackString() {
        $i = 0;
        $out = '';
        for($i=count($this->stack)-1; $i>=0; --$i) {
            $invocation = $this->stack[$i];
            $out .= $i.' '.$invocation->TaskType().' '.$invocation->TaskName()."\n";
        }
        return $out;
    }
}

// represents an invocation of a view, activity, or data collector

abstract class ACF_Invocation {
    private $taskName;
    private $parameterList;
    protected $taskType;

    function __construct($taskName, $parameterList) {
        $this->SetTaskName($taskName);
        $this->SetParameterList($parameterList);
//$this->ParameterList()->Dump('constructing '.$taskName);
    }

    function __clone() {
        $this->parameterList = clone $this->parameterList;
    }

    function SetParameterList(ACF_ParameterList $parameterList) {
        if($parameterList == NULL)
            throw new ACF_Exception(__METHOD__.': parameterList may be empty, but may not be NULL');
        $this->parameterList = $parameterList;
    }

    function SetTaskName($taskName) {
        $this->taskName = strtolower(trim($taskName));
        if(empty($this->taskName))
            throw new ACF_Exception(__METHOD__.': Cannot set null taskName');
    }

    function ParameterList() {
        return $this->parameterList;
    }

    function TaskName() {
        return $this->taskName;
    }

    function TaskType() {
        if(empty($this->taskType))
            throw new ACF_Exception('ACF_Invocation has no type');
        return $this->taskType;
    }

    // Return a list of parameters containing one firect parameter for each
    // invocation parameter.  Indirectn invocation parameters are resolved
    // from the provided $DirectParameterList

    function ExecutionParameters(ACF_ParameterList $DirectParameterList) {
        $executionParameters = clone $this->parameterList;
        $executionParameters->MakeDirect($DirectParameterList);
        return $executionParameters;
    }
}

class ACF_ViewInvocation extends ACF_Invocation {
    function __construct($name, $parameterList) {
        parent::__construct($name, $parameterList);
        $this->taskType = 'view';
    }
}

class ACF_ActivityInvocation extends ACF_Invocation {
    function __construct($name, $parameterList) {
        parent::__construct($name, $parameterList);
        $this->taskType = 'activity';
    }
}

class ACF_DataCollectorInvocation extends ACF_Invocation {
    function __construct($name, $parameterList) {
        parent::__construct($name, $parameterList);
        $this->taskType = 'datacollector';
    }
}

class ACF_InvocationList {
    private $invocations;

    function __construct() {
        $this->ClearList();
    }

    function __clone() {
        $this->invocations = CloneObjectArray($this->invocations);
    }

    function ClearList() {
        $this->invocations = array();
    }

    function AddInvocation(ACF_Invocation $invocation) {
        $this->invocations[$invocation->TaskName()] = $invocation;
    }

    function InvocationsArray() {
        return $this->invocations;
    }

    function ImportInvocationList($InvocationList,$ParentController) {
        $invocations = $InvocationList->InvocationsArray();
        if($invocations) {
            foreach($invocations as $invocation) {
                $newparameterList = $ParentController->CreateParameterList();
                $newparameterList->ImportParameterList($invocation->ParameterList());
                switch ($invocation->TaskType()) {
                    case 'view' :
                        $newinvocation = new ACF_ViewInvocation($invocation->TaskName(),$newparameterList);
                        break;
                    case 'activity' :
                        $newinvocation = new ACF_ActivityInvocation($invocation->TaskName(),$newparameterList);
                        break;
                    case 'datacollector' :
                        $newinvocation = new ACF_DataCollectorInvocation($invocation->TaskName(),$newparameterList);
                        break;
                }
                $this->AddInvocation($newinvocation);
            }
        }
    }

    function FindInvocation($TaskName) {
        return $this->invocations[$TaskName];
    }

    function MoveInvocationOrder($taskName,$direction) {
        $newInvocationArray = array();

        if($direction == 'up')
            $this->invocations = array_reverse($this->invocations);

        foreach($this->invocations as $key => $invocation) {

            if($key == $taskName)
                $drop = $invocation;
            else {
                $newInvocationArray[$key] = $invocation;
                if($drop) {
                    $newInvocationArray[$taskName] = $drop;
                    unset($drop);
                }
            }
        }

        if($direction == 'up')
            $newInvocationArray = array_reverse($newInvocationArray);

        $this->invocations = $newInvocationArray;
    }

    function RemoveInvocation($name) {
        unset($this->invocations[strtolower(trim($name))]);
    }
}

class ACF_Controller {
    protected $processManager;
    protected $activityManager;
    protected $executableWrapperManager;
    protected $viewDefinitionManager;
    protected $invocationStack;
    protected $managers;
    protected $layers;

    function __construct($initManagers, $layers = array('defaultlayer')) {
        $this->managers = array('viewDefinitionManager', 'processManager', 'executableWrapperManager', 'activityManager');

        $this->invocationStack = new ACF_InvocationStack();

        // Concrete class constructor should instantiate the managers
        if($initManagers) {
            $this->processManager = new ACF_ProcessManager($layers);
            $this->activityManager = new ACF_ActivityManager($layers);
            $this->executableWrapperManager = new ACF_ExecutableWrapperManager($layers);
            $this->viewDefinitionManager = new ACF_ViewDefinitionManager($layers);
        }
        $this->SetLayers($layers);
    }

    function ImportController($controller) {
        $this->processManager->ImportManager($controller->ProcessManager(),$this);
        $this->executableWrapperManager->ImportManager($controller->ExecutableWrapperManager());
        $this->viewDefinitionManager->ImportManager($controller->ViewDefinitionManager(),$this);
        $this->SetLayers($controller->Layers());
    }

    function SetLayers($layers= array('defaultlayer')) {
        $this->layers = array();
        foreach($layers as $layer)
            $this->layers[] = strtolower($layer);

        $this->layers = $layers;
        foreach($this->managers as $manager)
            if(!empty($this->$manager))
                $this->$manager->SetLayers($layers);
    }

    function Process(ACF_Request $request) {
        $event = $request->Event();
        $eventSource = $request->EventSource();
        $parameterList = $event->ParameterList();
        if($parameterList->Parameter('ACF_FLUSH')) {
            $this->ClearCache();
            $parameterList->DeleteParameterName('ACF_FLUSH');
        }

        do {
            $response = $this->processManager->ResponseTo($eventSource, $event);
            $this->invocationStack->LogEventToHistory($eventSource, $event);

            if(!$response) {    // Don't know what to do in response to this event
                $msg = __METHOD__.": Don't know how to respond to event ";
                $msg .= '"'.$event->Name().'" from ';
                $msg .= '"'.$eventSource->Realm().':'.$eventSource->Name()."\".\n";
                throw new ACF_Exception($msg);
            }
            $this->invocationStack->ProcessEventResponse($response);

            $invocation = $this->invocationStack->TopOfStack();
            if(!$invocation)
                throw new ACF_Exception('Invocation stack ran empty without returning a view.');

            if($invocation->TaskType() == 'view')
                break;
            $event = $this->ExecuteInvocation('event', $invocation, $parameterList);
            $eventSource = new ACF_EventSource('internal', $invocation->TaskName());
        } while(1);

        // Pop view off the stack
        $this->invocationStack->PopStack();

        $viewName = $invocation->TaskName();
        $viewDefinition = $this->viewDefinitionManager->ViewDefinitionFromInvocation($invocation);
        if(!$viewDefinition)
            throw new ACF_Exception("No definition found for view '$viewName'");

        $dataCollectors = $viewDefinition->DataCollectors()->InvocationsArray();

        // Run data collectors and add the parameters they return to the
        // Execution parameters.  The ecentresponse that invokes a view does *NOT*
        // need to be aware of parameters which required by the view and provided
        // by the data collectors.

        $executionParameters = $invocation->ExecutionParameters($parameterList);
        foreach($dataCollectors as $dataCollector) {
            // Note that parameters returned by the data collector are being added to
            // the execution parameter list rather than the global one.
            $this->ExecuteInvocation('parameterlist', $dataCollector, $executionParameters);
        }

        // Check parameters
        $viewDefinition->ValidateParameterList($executionParameters);

        return new ACF_ViewInvocation($viewName, $executionParameters);
    }

    function EnableStackHistory($arg) {
        $this->invocationStack->History($arg ? true : false);
    }

    function StackHistoryString() {
        return $this->invocationStack->HistoryString();
    }

    function StackString() {
        return $this->invocationStack->StackString();
    }

    function ExecuteInvocation($expectedReturnType, ACF_Invocation $invocation, ACF_ParameterList $parameterList) {
        // Execute Activity    or data collector
        $taskType = $invocation->TaskType();
        $taskName = $invocation->TaskName();

        $executableWrapper = $this->executableWrapperManager->ExecutableWrapperFromInvocation($invocation);
        if(!$executableWrapper) {
            $msg = "Cannot find executable for $taskType $taskName";
            throw new ACF_Exception($msg);
        }

        $returnType = $executableWrapper->ReturnType();
        if($returnType != $expectedReturnType){
            $name = $executableWrapper->Name();
            throw new ACF_Exception("Controller wants '$expectedReturnType', but executable $name returns '$returnType'");
        }

        try {
            $executionParameters = $invocation->ExecutionParameters($parameterList);
            $output = $executableWrapper->Execute($executionParameters);
        } catch (ACF_Exception $e) {
            throw new ACF_Exception("Problem executing  $taskType $taskName: ".$e->GetMessage());
        }

        if($output instanceof ACF_ParameterList) {
            $parameterList->AddParameterList($output);

        } else if($output instanceof ACF_Event) {
            $parameterList->AddParameterList($output->ParameterList());

        } else {    // Don't know how to handle this
            $type = gettype($output);
            if($type == 'object')
                $type = get_class($output).' object';
            throw new ACF_Exception(__METHOD__.": Don't know how to handle $type returned by $taskType $taskName.");
        }

        return $output;
    }

    // If persistent configuration has changed, call ClearCache to clear any
    // portion of it that has been cached by a caching child.
    // cacheType may be process, activity, executable, process.
    // If cacheType==NULL, clear all caches.

    function ClearCache($cacheType=NULL) {
        if(!$cacheType) {
            foreach($this->managers as $manager) {
                if(!empty($this->$manager))
                    $this->$manager->ClearCache();
            }
            return;
        }

        $cacheType .= 'Manager';
        $this->$cacheType->ClearCache();
    }

    function ProcessManager() {
        return $this->processManager;
    }

    function ExecutableWrapperManager() {
        return $this->executableWrapperManager;
    }

    function ViewDefinitionManager() {
        return $this->viewDefinitionManager;
    }

    function Layers() {
        return $this->layers;
    }
}

?>
