<?PHP

// This class represents executables that may be invoked by the ACF.
// including decoders, encoders, activity handlers, view handlers,
// and data collectors

abstract class ACF_ExecutableWrapper {
	private $returnedEventNames;
	private $name;
	private $returnType;
	protected $callType;
	private $names;

	// Caching Properties (Added By JCW 02/04/05)
	private $cacheTimeout; // Max age of a value set
	private $cacheValues; // Array of Input/Output Values w/ timestamps
	private $cacheGlobal; // Boolean flag to use session or file.
	private $cacheFilename; // Filename for global file.
	public $projectIdentifier; // Project ID for Filename
	
	function __construct($name, $returnType)
	{
		$this->callType = NULL;
		$this->SetName($name);
		$this->SetReturnType($returnType);
		$this->SetOutputEvents(array());
		$this->SetRequiredInputParameters(array());
		$this->SetOptionalInputParameters(array());
		$this->SetRequiredOutputParameters(array());
		$this->SetOptionalOutputParameters(array());

		// Caching Options
		$this->cacheTimeout = 0;
		$this->cacheGlobal = false;
	}
	
	function SetCachingOptions($timeout=60,$globalSwitch = false) { // Caching (Added By JCW 02/04/05)
		$this->cacheTimeout = $timeout;
		$this->cacheGlobal = $globalSwitch;
	}
	
	function SetReturnType($returnType) {
		if(!$returnType)
			throw new ACF_Exception(__METHOD__.': may not set null return type');
		if(($returnType!= 'parameterlist') && ($returnType!= 'event'))
			throw new ACF_Exception(__METHOD__.": Invalid return type '$returnType'");
		$this->returnType = $returnType;
	}
	
	private function AddName($type, $name)
	{
		$name = strtolower(trim($name));
		if(!IsValidName($name))
			throw new ACF_Exception("Invalid charactern in parameter name '$name'.");
		$this->names[$type][] = $name;
	}
	
	/*
	 * Set list of names of optional or required input or output parameters.  Accepts array or string containing
	 * comma separated list with optional whitespace (IE 'name1, name2, name3')
	 *
	 */
	 
	private function SetNames($type, $names)
	{
		$this->names[$type] = array();
		
		$parameterType = gettype($names);
		if(gettype($names)=='string') {	// Explode into array
			$names = explode(',', $names);
		} else if($parameterType != 'array')
			throw new ACF_Exception("Invalid parameter type: $parameterType.");
		
		foreach($names as $name) {
			$name = trim($name);
			if(empty($name))
				continue;
			$this->AddName($type, $name);
		}
	}

	function Names($type) { // Added for persistence (JCW 8/10/04)
		return $this->names[$type];
	}
	
	/*
	 * Set / Add input parameters
	 *
	 */
	 
	function SetRequiredInputParameters($names) 	{
		$this->SetNames('required_input', $names);
	}

	function AddRequiredInputParameter($name) 	{
		$this->AddName('required_input', $name);
	}
	
	function SetOptionalInputParameters($names) 	{
		$this->SetNames('optional_input', $names);
	}
	
	function AddOptionalInputParameter($name) 	{
		$this->AddName('optional_input', $name);
	}

	/*
	 * Set / Add input parameters
	 *
	 */
	 
	function SetRequiredOutputParameters($names) 	{
		$this->SetNames('required_output', $names);
	}
	
	function AddRequiredOutputParameter($name) 	{
		$this->AddName('required_output', $name);
	}
	
	/**
	* @return void
	* @param unknown $names
	* @desc Enter description here...
	*/
	function SetOptionalOutputParameters($names) 	{
		$this->SetNames('optional_output', $names);
	}
	
	function AddOptionalOutputParameter($name) 	{
		$this->AddName('optional_output', $name);
	}	
	
	function SetName($name) {
		$this->name = strtolower(trim($name));
	}

	function Name() {
		return $this->name;
	}
	
	function ReturnType() {
		return $this->returnType;
	}
	
	function CallType()
	{
		if(!$this->callType)
			throw new ACF_Exception('CallType undefined in ACF_ExecutableWrapper');
		return $this->callType;
	}

	function CacheTimeout()
	{
		return $this->cacheTimeout; 
	}
	
	function CacheGlobal()
	{
		return $this->cacheGlobal; 
	}

	function Execute(ACF_ParameterList $parameterList)
	{
		$foundCache = false;
		$inputParameterList;

		// Check that all required parameters have been provided
		foreach($this->names['required_input'] as $name)
			if(!$parameterList->Parameter($name))
				throw new ACF_Exception("Required parameter '$name' not present in executable ".$this->Name());
		
		// Check that all provided parameters are acceptable
		$parameterNames = $parameterList->Names();
		foreach($parameterNames as $name) {
			if(in_array($name, $this->names['required_input']))	// Is a required parameter
				continue;
			if(in_array($name, $this->names['optional_input']))	// is an optional parameter
				continue;
			
			// Is not an expected parameter
			throw new ACF_Exception('Executable '.$this->Name()." passed unrecognized parameter name '$name'");
		}

		// Check Cache
		if($this->cacheTimeout > 0) {
			if($this->cacheGlobal) { // Check Global Cache Option
				$result = $this->checkGlobalCache($parameterList);	
			} else {
				$result = $this->checkLocalCache($parameterList);	
			}

			$inputParameterList = $parameterList; // Keep a pointer on this list for Caching
		}

		if(!$result) // (Added By JCW 02/04/05) 
			$result = $this->_Execute($parameterList);	// Do it!
		else
			$foundCache = true;

		// Check for valid return
		if($this->returnType == 'event') {
			if(!($result instanceof ACF_Event))
				throw new ACF_Exception(get_class($this).' executable '.$this->Name().' returned something other than an event.');
			if(!$this->IsPermittedOutputEvent($result->Name()))
				throw new ACF_Exception(get_class($this).' executable '.$this->Name().' returned invalid event '.$result->Name());
		} else 	if($this->returnType == 'parameterlist') {
			if(!($result instanceof ACF_ParameterList))
				throw new ACF_Exception(get_class($this).' executable '.$this->Name().' returned something other than a parameterlist.');
		} else {
			throw new ACF_Exception(get_class($this).' executable '.$this->Name()." has unrecognized return type '$returnType'");
		}
		
		// Now look at returned parameters
		$parameterList = $this->returnType=='event' ? $result->ParameterList() : $result;
		foreach($parameterList->Names() as $name) {
			if($this->IsPermittedOutputParameter($name))
				continue;
			throw new ACF_Exception('Executable '.$this->Name()." returned unexpected parameter named '$name'");
		}

		if(!$foundCache && $inputParameterList && !$this->cacheGlobal) // (Added By JCW 02/04/05) 
			$this->storeLocalCache($inputParameterList, $result);
		else if(!$foundCache && $inputParameterList) // (Added By JCW 02/07/05) 
			$this->storeGlobalCache($inputParameterList, $result);

		return $result;
	}
	
	abstract protected function _Execute(ACF_ParameterList $parameters);
	abstract function Validate();	// Return NULL if okay, else error message
	
	function SetOutputEvents($names) 	{
		$this->SetNames('output_event', $names);
	}
	
	function AddOutputEvent($name)
	{
		$this->AddName('output_event', $name);
	}
	
	function IsPermittedOutputEvent($name) {
		return $this->IsPermittedName('output_event', $name);		
	}
	
	function IsPermittedOutputParameter($name) {
		if($this->IsPermittedName('required_output', $name))
			return true;
		return $this->IsPermittedName('optional_output', $name);		
	}
	
	protected function IsPermittedName($type, $name)
	{
		return in_array(strtolower(trim($name)),$this->names[$type]);
	}

	function generateCacheIndex(ACF_ParameterList $parameterList) // (Added By JCW 02/04/05) 
	{
		$tempString = '';
		$tempString = $this->projectIdentifier.'-'.$this->name.'-'.$this->callType.'-'.serialize($parameterList); // Convert Object To String
		return md5($tempString);
	}

	function checkLocalCache(ACF_ParameterList $parameterList) // (Added By JCW 02/04/05) 
	{
		$cachedTime; 
		$cachedValue;
		$indexString;

		//Generate Index
		$indexString = $this->generateCacheIndex($parameterList);

		$cachedValue = $this->cacheValues[$indexString];

		if($cachedValue) { // Found a Match, compare values
			$cachedTime = $this->cacheValues[$indexString]['tstamp'];
			if((time() - $cachedTime) <= $this->cacheTimeout) // Still Fresh Data
				return $this->cacheValues[$indexString]['result'];
			else // Outdated, Flush Entry
				unset($this->cacheValues[$indexString]);
		}
		
		return;
	}

	function storeLocalCache(ACF_ParameterList $input, $result) // (Added By JCW 02/04/05) 
	{
		// result is not typed because one could cache activityhandlers with a few modifications
		
		$indexString = '';

		$indexString = $this->generateCacheIndex($input);
		$this->cacheValues[$indexString] = array();
		$this->cacheValues[$indexString]['tstamp'] = time();
		$this->cacheValues[$indexString]['result'] = clone($result);
	}
	
	function checkGlobalCache(ACF_ParameterList $parameterList) // (Added By JCW 02/07/05) 
	{
		$filename = '';
		$fr; // File Resource
		$cachedTime;
		$objectString;
		global $ACF_CacheDir;

		$filename = $ACF_CacheDir.'/'.$this->generateCacheIndex($parameterList);
		if(file_exists($filename) && $fr = fopen($filename,"r")) {
			$cachedTime= intval(fgets($fr));
			$objectString = fgets($fr);
			fclose($fr);

			if((time() - $cachedTime) <= $this->cacheTimeout) {// Still Fresh Data
				return unserialize($objectString);
			} else { // Outdated, Flush File
				unlink($filename);
			}
		}
		
		return;
	}

	function storeGlobalCache(ACF_ParameterList $input, $result) // (Added By JCW 02/07/05) 
	{
		// result is not typed because one could cache activityhandlers with a few modifications
		$filename = '';
		$fr; // File Resource
		global $ACF_CacheDir;

		// Create New File (This may need Thread Protection, x will return false if exists but may spit up an ugly warning)
		$filename = $ACF_CacheDir.'/'.$this->generateCacheIndex($input);
		if($fr = fopen($filename,"x")) {
			// Dump Info
			fwrite($fr,time()."\r\n");
			fwrite($fr,serialize($result));
			fclose($fr);
		}
	}
}

class ACF_ExecutableWrapperManager {
	protected $wrappers;
	private $layers;
	
	function __construct($layers = array('defaultlayer'))
	{
		$this->ClearCache();
		$this->SetLayers($layers);
	}
	
	function ClearCache()
	{
		$wrappers = array();
	}

	function ImportManager($ExecManager) {
		$this->wrappers = $ExecManager->Wrappers();
		$this->$layers = $ExecManager->Layers();
	}
	
	function UpdateExecutable(ACF_ExecutableWrapper $wrapper, $layer='defaultlayer')
	{
		$this->AddExecutableWrapper($wrapper, $layers);
	}
	
	function AddExecutableWrapper(ACF_ExecutableWrapper $wrapper, $layer='defaultlayer')
	{
		$wrapper->Validate();
			
		$layer = strtolower(trim($layer));
		if(!in_array($layer, $this->layers))
			throw new ACF_Exception("Invalid layer name '$layer'");
		$lname = strtolower(trim($wrapper->Name()));

		$this->wrappers[$layer][$lname] = $wrapper;
	}
	
	function DeleteWrapper(ACF_ExecutableWrapper $wrapper, $layer='defaultlayer')
	{
		unset($this->wrappers[$layer][$wrapper->Name()]);
	}
	
	function ExecutableWrapperFromInvocation(ACF_Invocation $invocation)
	{
		$name = $invocation->TaskName();
			
		foreach($this->layers as $layer)
			if(!empty($this->wrappers[$layer][$name]))
				return $this->wrappers[$layer][$name];
		return NULL;
	}
		
	function Execute(ACF_Executable $wrapper)
	{
		$event = $wrapper->Execute($parameters);
	}
	
	function SetLayers($layers)
	{
		$this->layers = array();
		foreach($layers as $layer)
			$this->layers[] = strtolower($layer);
	}

	function Layers() {
	    return $this->layers;
	}

	function Wrappers() { // Add by JCW 8/24/04
	    return $this->wrappers;
	}
	
	function FindWrapper($layer,$name) { // Add by JCW 8/24/04
	    $llayer = strtolower($layer);
	    $lname = strtolower($name);
	    return $this->wrappers[$llayer][$lname];
	}
	
	function MoveWrapper($layer,$OldName,$NewName) { // Added by JCW 8/24/04
		$llayer = strtolower($layer);
		$loName = strtolower($OldName);
		$lnName = strtolower($NewName);

		if(isset($this->wrappers[$llayer][$loName])) {
			$this->wrappers[$llayer][$lnName] = $this->wrappers[$llayer][$loName];
			
			// Remove Old Value
			unset($this->wrappers[$llayer][$loName]);
		}
	}
}
?>
