<?php

abstract class ACF_Parameter {
	private $name;
	
	abstract function IsDirect();
	
	function SetName($name) {
		$this->name = strtolower($name);
	}
	
	function Name() {
		return $this->name;
	}
}


class ACF_DirectParameter extends ACF_Parameter  {
	private $value;
	
	function __construct($name, $value)
	{
		$this->SetName($name);
		$this->SetValue($value);
	}
	
	function SetValue($value)
	{
		$this->value = $value;
	}
	
	function IsDirect() {
		return true;
	}
	
	function IsOptional() { // Added Optional (JCW 8/12/04)
		return false; // Direct Parameters are always required
	}
	
	function Value() {
		return $this->value;
	}
}

class ACF_IndirectParameter extends ACF_Parameter {
	private $parameterName;
	private $optional; // Added Optional (JCW 8/12/04)
	
	function __construct($name, $reference, $optional=false)
	{
		$this->SetName($name);
		$this->SetReference($reference);
		$this->optional = $optional; // Added Optional (JCW 8/12/04)
	}
	
	function SetReference($reference)
	{
		$this->reference = $reference;
	}
	
	function Reference() {
		return $this->reference;
	}
	
	function IsDirect() {
		return false;
	}
	
	function IsOptional() { // Added Optional (JCW 8/12/04)
		return $this->optional;
	}
}

class ACF_ParameterList {
	private $parameters;
	
	function __construct()
	{
		$this->ClearList();
	}
	
	function __clone()
	{
		$this->parameters = CloneObjectArray($this->parameters);
	}
	
	function ClearList()
	{
		$this->parameters = array();
	}
	
	function AddParameter(ACF_Parameter $parameter)
	{
		$this->parameters[$parameter->Name()] = $parameter;
	}
	
	function AddParameterArray($parameters)
	{
		if(!is_array($parameters))
			throw new ACF_Exception('ACF_ParameterList::AddParameters called with a non-array parameter');
		foreach($parameters as $name=>$parameter)
			$this->AddParameter($parameter);
	}
	
	function DeleteParameterName($name)
	{
		unset($this->parameters[strtolower(trim($name))]);
	}
	
	function AddParameterList(ACF_ParameterList $parameterList)
	{
		$this->AddParameterArray($parameterList->ParametersArray());
	}
	
	function Parameter($name)
	{
		$name = strtolower(trim($name));
		if(!isset($this->parameters[$name]))
			return NULL;
		return $this->parameters[$name];
	}
	
	function ParameterExists($name) {
		$name = strtolower(trim($name));
		return isset($this->parameters[$name]);
	}
	
	function ParameterValue($name)
	{
		$name = strtolower($name);
		if(!isset($this->parameters[$name]))
			return NULL;
		return $this->parameters[$name]->Value();
	}
	
	function ParametersArray()
	{
		return $this->parameters;
	}

	function ImportParameterList($ParamList) {
		$this->parameters = $ParamList->ParametersArray();
	}
	
	function Names()	// Return an array of parameter names
	{
		$a = array();
		foreach($this->parameters as $parameter)
			$a[] = $parameter->Name();
		return $a;
	}
	
	function Dump($label)	// Return an array of parameter names
	{
		echo "<p># Dumping $label<br>";		
		foreach($this->parameters as $parameter) {
			echo '# &nbsp&nbsp&nbsp ';
			if($parameter->IsDirect())
				echo $parameter->Name().'  == "'.$parameter->Value().'"<br>';
			else
				echo $parameter->Name().'  => '.$parameter->Reference().'<br>';
		}
		echo "# End of dump</p>";
	}
	/*
	 * Make sure that all my parameters are now constants by dereferencing
	 * from the provided list of direct parameters.
	 *
	 */
	 
	function MakeDirect(ACF_ParameterList $directParameters)
	{
		foreach($this->parameters as $name=>$parameter) {
			if(!$parameter->IsDirect()) {
				$newName = $parameter->Reference();
				$newParameter = $directParameters->Parameter($newName);
				if(!$newParameter && !$parameter->IsOptional()) // Added Optional (JCW 8/12/04)
					throw new ACF_Exception("Parameter '$name' refers to unknown parameter '$newName'");
				else if($newParameter && !$newParameter->IsDirect())
					throw new ACF_Exception(__METHOD__.":Provided parameter '$name' is also indirect.");
				else if($newParameter)
					$this->parameters[$name] = $newParameter;
				else
					unset($this->parameters[$name]);
			}
		}
		// All my parameters are now direct (Except optional ones JCW 8/12/04)
	}
}

?>
