<?php

class ACF_ViewDefinition {
	private $name;
	private $dataCollectors;
	private $names;
	
	function __construct($name)
	{
		$this->SetName($name);
		$this->SetRequiredInputParameters(array());
		$this->SetOptionalInputParameters(array());
		$this->ClearDataCollectors();
	}
	
	function SetName($name)
	{
		$name = strtolower(trim($name));
		if(empty($name))
			throw new ACF_Exception('Attempt to give view a null name');
		$this->name = $name;
	}
	
	function Name() {
		return $this->name;
	}
	
	function ClearDataCollectors()
	{
		$this->dataCollectors = array();
	}
	
	function DataCollectors()
	{
		return $this->dataCollectors;
	}
	
	function AddDataCollector(ACF_DataCollectorInvocation $dataCollector)
	{
		if(!$dataCollector)
			throw new ACF_Exception('Attemptred to add null data collector to view definition "'.$this->name.'"');
		$this->dataCollectors[] = $dataCollector;	
	}
	
	function SetDataCollectorList(ACF_InvocationList $invocationList)
	{
		$this->dataCollectors = $invocationList;
	}
	
	/*
	 * Set list of names of optional or required input or output parameters.  Accepts array or string containing
	 * comma separated list with optional whitespace (IE 'name1, name2, name3')
	 *
	 */
	 
	private function SetNames($type, $names)
	{
		$this->names[$type] = array();
		
		$parameterType = gettype($names);
		if(gettype($names)=='string') {	// Explode into array
			$names = explode(',', $names);
		} else if($parameterType != 'array')
			throw new ACF_Exception("Invalid parameter type: $parameterType.");
		
		foreach($names as $name) {
			$name = trim($name);
			if(empty($name))
				continue;
			$this->AddName($type, $name);
		}
	}

	function Names($type) { // Added for persistence (JCW 8/10/04)
                return $this->names[$type];
        }
	
	private function AddName($type, $name)
	{
		$name = strtolower(trim($name));
		if(!IsValidName($name))
			throw new ACF_Exception("Invalid charactern in parameter name '$name'.");
		$this->names[$type][] = $name;
	}
	
	/*
	 * Set / Add input parameters
	 *
	 */
	 
	function SetRequiredInputParameters($names) 	{
		$this->SetNames('required_input', $names);
	}
	
	function AddRequiredInputParameter($name) 	{
		$this->AddName('required_input', $name);
	}
	
	function SetOptionalInputParameters($names) 	{
		$this->SetNames('optional_input', $names);
	}
	
	function AddOptionalInputParameter($name) 	{
		$this->AddName('optional_input', $name);
	}
	
		function IsPermittedOutputEvent($name) {
		return $this->IsPermittedName('output_event', $name);		
	}
	
	function IsPermittedInputParameter($name) {
		if($this->IsPermittedName('required_input', $name))
			return true;
		return $this->IsPermittedName('optional_input', $name);		
	}
	
	protected function IsPermittedName($type, $name)
	{
		return in_array(strtolower(trim($name)),$this->names[$type]);
	}
	
	function Validate()
	{
		if(empty($this->name))
			throw new ACF_Exception('NULL name in '.__CLASS__);
	}
	
	// Check that all parameters required by this view are present
	
	function ValidateParameterList(ACF_ParameterList $parameterList)
	{
		$errors = array();
		foreach($this->names['required_input'] as $name)
			if(!$parameterList->ParameterExists($name))
				$errors[] = $name;
				
		if(count($errors)) {
			$names = '';
			foreach($errors as $name)
				if($names)
					$names .= ', '.$name;
				else
					$names = $name;
			throw new ACF_Exception("Required parameter(s) ($names) not present for invocation of view ".$this->Name());
		}
		
		// Check that all provided parameters are acceptable
		$parameterNames = $parameterList->Names();
		foreach($parameterNames as $name) {
			if(in_array($name, $this->names['required_input']))	// Is a required parameter
				continue;
			if(in_array($name, $this->names['optional_input']))	// is an optional parameter
				continue;
			
			// Is not an expected parameter
			throw new ACF_Exception('View '.$this->Name()." passed unrecognized parameter name '$name'");
		}
	}
}

class ACF_ViewDefinitionManager  {
	private $layers;
	private $definitions;
	
	function __construct($layers= array('defaultlayer'))
	{
		$this->ClearCache();
		$this->SetLayers($layers);
	}
		
	function SetLayers($layers)
	{
		$this->layers = array();
		foreach($layers as $layer)
			$this->layers[] = strtolower($layer);
	}
	
	function ClearCache()
	{
		$this->definitions = array();
	}
	
	function ImportManager($ViewManager,$ParentController) {
		$definitions = $ViewManager->Definitions();
		if($definitions) {
			foreach($definitions as $layer => $defs) {
				foreach($defs as $definition) {
					$newdefinition = new ACF_ViewDefinition($definition->Name());
					$newinvocationList = $ParentController->CreateInvocationList();
					$newinvocationList->ImportInvocationList($definition->DataCollectors(),$ParentController);
					$newdefinition->SetDataCollectorList($newinvocationList);
					$newdefinition->SetRequiredInputParameters($definition->Names('required_input'));
					$newdefinition->SetOptionalInputParameters($definition->Names('optional_input'));
					$this->definitions[$layer][$newdefinition->Name()] = $newdefinition;
				}
			}
		}
	}

	function AddViewDefinition(ACF_ViewDefinition $definition, $layer='defaultlayer')
	{
		$name = $definition->Name();
		$definition->Validate();
				
		$layer = strtolower(trim($layer));
		if(!in_array($layer, $this->layers))
			throw new ACF_Exception("Invalid layer name '$layer'");
		$this->definitions[$layer][$name] = $definition;
	}
	
	function DeleteViewDefinition(ACF_ViewDefinition $definition, $layer='defaultlayer')
	{
		unset($this->definitions[$layer][$definition->Name()]);
	}
	
	function ViewDefinitionFromInvocation(ACF_ViewInvocation $invocation)
	{
		$name = $invocation->TaskName();
			
		foreach($this->layers as $layer)
			if(!empty($this->definitions[$layer][$name]))
				return $this->definitions[$layer][$name];
		return NULL;
	}

	function Definitions() { // Added by JCW 8/24/04
	    return $this->definitions;
	}
	
	function FindDefinition($layer,$name) { // Added by JCW 8/24/04
	    $llayer = strtolower($layer);
	    $lname = strtolower($name);
	    return $this->definitions[$llayer][$lname];
	}
	
	function MoveDefinition($layer,$OldName,$NewName) { // Added by JCW 8/24/04
		$llayer = strtolower($layer);
		$loName = strtolower($OldName);
		$lnName = strtolower($NewName);

		if(isset($this->definitions[$llayer][$loName])) {
			$this->definitions[$llayer][$lnName] = $this->definitions[$llayer][$loName];
			
			// Remove Old Value
			unset($this->definitions[$llayer][$loName]);
		}
	}
}
?>
