<?php

require_once($_ENV['TCA_HOME']."/util/parameter.inc.php");
require_once($_ENV['TCA_HOME']."/cartridges/parameter_validation.inc.php");
require_once($_ENV['TCA_HOME']."/cartridges/enumeration.inc.php");

interface TCA_ActionBasicParameterI {
    function getName();

    function getDescription();

    function getUsage();

    function isRequired();

    // returns TCA_ActionParameterValidationResultI
    function validate();

    function getValidator();

    function isEmpty();
}

abstract class TCA_AbstractBasicParameter {
    private $name;
    private $description;
    private $usage;
    private $isRequired;
    protected $validator;

    function __construct($name, $description, $usage, $isRequired, TCA_ActionParameterValidatorI $validator) {
        $this->name = $name;
        $this->description = $description;
        $this->usage = $usage;
        $this->isRequired = $isRequired;
        $this->validator = $validator;
    }

    function getName() {
        return $this->name;
    }
    
    function getDescription() {
        return $this->description;
    }
    
    function getUsage() {
        return $this->usage;
    }

    function isRequired() {
        return $this->isRequired;
    }        

    function getValidator() {
        return $this->validator;
    }    
    
    function __toString() {
        return "'" . $this->name . "' => ";
    }
}

interface TCA_ActionValueParameterI extends TCA_ActionBasicParameterI {
    function getValue();
    
    function setValue($value);
}

class TCA_ActionValueParameter extends TCA_AbstractBasicParameter implements TCA_ActionValueParameterI {
    private $value;

    function setValue($value) {
        $this->value = $value;
    }
    
    function getValue() {
        return $this->value;
    }
    
    function validate() {
        if ($this->isRequired() && !isset($this->value)) {
            $result = new TCA_ActionParameterValidationResult(false, "Required parameter is not set");
        } else {
            $result = $this->validator->validate($this->value);
        }
        return $result;
    }

    function isEmpty() {
        return isset($this->value) ? false : true;
    }
    
    function __toString() {
        return parent::__toString() . (isset($this->value) ? "'" . addcslashes($this->value, "'") . "'" : 'undef');
    }
}

class TCA_ActionBooleanParameter extends TCA_ActionValueParameter implements TCA_ActionValueParameterI {
    function __construct($name, $description, $usage, $isRequired) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_BooleanValidationRule()));
    }
}

class TCA_ActionNumericParameter extends TCA_ActionValueParameter implements TCA_ActionValueParameterI {
    function __construct($name, $description, $usage, $isRequired, $min, $max) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_NumericRangeValidationRule($min, $max)));
    }    
}

class TCA_ActionIntegerParameter extends TCA_ActionValueParameter implements TCA_ActionValueParameterI {
    function __construct($name, $description, $usage, $isRequired, $min, $max) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_IntegerRangeValidationRule($min, $max)));
    }
    
    function setValue($value) {
        // force type conversion
        parent::setValue(intval($value));
    }
}

class TCA_ActionStringParameter extends TCA_ActionValueParameter implements TCA_ActionValueParameterI {	
    function __construct($name, $description, $usage, $isRequired, $regExp, $regExpDescription) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_RegExpValidationRule($regExp, $regExpDescription)));
    }
}

class TCA_ActionEnumeratedParameter extends TCA_ActionValueParameter implements TCA_ActionValueParameterI {
    function __construct($name, $description, $usage, $isRequired, TCA_Enumeration $enumeration) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_EnumeratedValidationRule($enumeration)));
    }
}

////////////////// values /////////////////////

interface TCA_ActionValuesParameterI extends TCA_ActionBasicParameterI {
    function getValues();
    
    function addValue($value);
    
    function addValues($values);
}

class TCA_ActionValuesParameter extends TCA_AbstractBasicParameter implements TCA_ActionValuesParameterI {
    private $values = array();

    function addValue($value) {
        $this->values[] = $value;
    }

    function addValues($values) {
        array_push($this->values, $values);
    }
    
    function getValues() {
        return $this->values;
    }
        
    function validate() {
        $validationResults = array();
        
        if ($this->isRequired() && count($this->values) == 0) {
            $validationResults[] = new TCA_ActionParameterValidationResult(false, "Required parameter is not set");
        } else {    	
            foreach ($this->values as $value) {
                $result = $this->validator->validate($value);	    		
                $errorDescription = "";
                if (!$result->isValid()) {
                    $errorDescription = "'" . $value . "' value is not valid.";
                    $result = new TCA_ActionParameterValidationResult($result->isValid(), $errorDescription);
                }
                array_push($validationResults, $result);
            }
        }
        return $validationResults;
    }
    
    function isEmpty() {
        return sizeof($this->values) > 0 ? false : true;
    }
    
    function __toString() {
        return parent::__toString() . "[" . implode(',', $this->values) . "]";
    }
}

class TCA_ActionBooleanSetParameter extends TCA_ActionValuesParameter implements TCA_ActionValuesParameterI {
    function __construct($name, $description, $usage, $isRequired) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_BooleanValidationRule()));
    }
}

class TCA_ActionNumericSetParameter extends TCA_ActionValuesParameter implements TCA_ActionValuesParameterI {
    function __construct($name, $description, $usage, $isRequired, $min, $max) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_NumericRangeValidationRule($min, $max)));
    }
}

class TCA_ActionIntegerSetParameter extends TCA_ActionValuesParameter implements TCA_ActionValuesParameterI {
    function __construct($name, $description, $usage, $isRequired, $min, $max) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_IntegerRangeValidationRule($min, $max)));
    }
}

class TCA_ActionStringSetParameter extends TCA_ActionValuesParameter implements TCA_ActionValuesParameterI {	
    function __construct($name, $description, $usage, $isRequired, $regExp, $regExpDescription) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_RegExpValidationRule($regExp, $regExpDescription)));
    }
}

class TCA_ActionEnumSetParameter extends TCA_ActionValuesParameter implements TCA_ActionValuesParameterI {
    function __construct($name, $description, $usage, $isRequired, TCA_Enumeration $enumeration) {
        parent::__construct($name, $description, $usage, $isRequired,
            new TCA_GenericValidator(new TCA_EnumeratedValidationRule($enumeration)));
    }
}

interface TCA_ActionParameterContainerI {
    function getName();

    function addParameter(TCA_ActionBasicParameterI $parameter);
    
    // returns TCA_ActionBasicParameterI[]
    function getParameters();

    function getParameterByName($name);
    
    // validate container's parameters - returns TCA_ActionParameterValidationResultI[]
    public function validate();

    function addChildContainer(TCA_ActionParameterContainerI $container);
    
    function getChildContainerByName($name);
    
    function removeChildContainerByName($name);
        
    // returns TCA_ActionParameterContainerI[]
    function getChildContainers();    
    
    // returns TCA_ParameterSet
    function toParameterSet();    
}

class TCA_ActionParameterContainer implements TCA_ActionParameterContainerI {
    const IDENT_STR = "    ";
    private $name;
    private $parameters = array();
    private $childContainers = array();

    function __construct($name) {    	
        $this->name = $name;
    }
    
    function getName() {
        return $this->name;
    }
    
    function addParameter(TCA_ActionBasicParameterI $parameter) {
        $this->parameters[$parameter->getName()] = $parameter;
    }
    
    function getParameters() {
        return array_values($this->parameters);
    }

    function getParameterByName($name) {
        return $this->parameters[$name];
    }
    
    function validate() {
        $results = array();
        foreach ($parameters as $param) {
            $results[] = $param->validate();
        }
        return $results;
    }
        
    function addChildContainer(TCA_ActionParameterContainerI $container) {
        $this->childContainers[$container->getName()] = $container;
    }

    function getChildContainerByName($name) {
        return $this->childContainers[$name];
    }

    function removeChildContainerByName($name) {
        unset($this->childContainers[$name]);
    }
    
    function getChildContainers() {
        return array_values($this->childContainers);
    }
    
    function toParameterSet() {
        $parameterSet = new TCA_ParameterSet();
        foreach ($this->getParameters() as $actionParam) {
            // skip empty (aka unset) parameters
            if ($actionParam->isEmpty()) {
                continue;
            }
            if ($actionParam instanceof TCA_ActionValueParameterI) {
                $parameter = new TCA_Parameter($actionParam->getName(), $actionParam->getValue());
                $parameterSet->addParameter($parameter);
            } elseif ($actionParam instanceof TCA_ActionValuesParameterI) {
                $parameterSet->fromArray($actionParam->getValues());
            } else {
                throw new Exception("Invalid parameter type");
            }
        }
        
        foreach ($this->getChildContainers() as $childContainer) {
            $parameterSet->addParameterSet($childContainer->getName(), $childContainer->toParameterSet());
        }
        
        return $parameterSet;
    }

    function __toString() {
        return __toString_internal();
    }

    protected function __toString_internal($ident = 0) {
        $identStr = str_repeat(self::IDENT_STR, $ident);
        $out = $identStr . "'" . $this->name . "' => {\n";
        foreach ($this->parameters as $parameter) {
            $out .= $identStr . self::IDENT_STR . $parameter->__toString() . ",\n";
        }
        foreach ($this->childContainers as $childContainer) {
            $out .= $childContainer->__toString_internal($ident + 1);
        }
        $out .= $identStr . "},\n";

        return $out;
    }
}

interface TCA_ActionResultI {
    function isSuccess();
    
    function getResultCode();
    
    function getResultText();
    
    // returns TCA_ActionParameterContainerI
    function getResultData();
}

class TCA_ActionResult implements TCA_ActionResultI {
    private $isSuccess;
    private $code;
    private $text;
    private $dataContainer;
    
    function __construct($isSuccess, $code, $text) {
        $this->isSuccess = $isSuccess;
        $this->code = $code;
        $this->text = $text;
        $this->dataContainer = new TCA_ActionParameterContainer("result");
    }
    
    function isSuccess() {
        return $this->isSuccess;
    }
    
    function getResultCode() {
        return $this->code;
    }
    
    function getResultText() {
        return $this->text;
    }
    
    function getResultData() {
        return $this->dataContainer;
    }
    
    function __toString() {
        return	"isSuccess: " . $this->isSuccess . "\n" .
            "Result Code: " . $this->code . "\n" .
            "Result Text: " . $this->text . "\n" .
            $this->dataContainer->__toString();
    }
}

class TCA_ActionParameterFactory {
    static function newBooleanParameter($name, $description, $usage, $isRequired) {
        return new TCA_ActionBooleanParameter($name, $description, $usage, $isRequired);
    }
    
    static function newNumericParameter($name, $description, $usage, $isRequired, $min, $max) {
        return new TCA_ActionNumericParameter($name, $description, $usage, $isRequired, $min, $max);
    }
    
    static function newIntegerParameter($name, $description, $usage, $isRequired, $min, $max) {
        return new TCA_ActionIntegerParameter($name, $description, $usage, $isRequired, $min, $max);
    }
    
    static function newStringParameter($name, $description, $usage, $isRequired, $regExp, $regExpDescription) {
        return new TCA_ActionStringParameter($name, $description, $usage, $isRequired, $regExp, $regExpDescription);
    }
    
    static function newEnumeratedParameter($name, $description, $usage, $isRequired, TCA_Enumeration $enumeration) {
        return new TCA_ActionEnumeratedParameter($name, $description, $usage, $isRequired, $enumeration);
    }

    static function newBooleanSetParameter($name, $description, $usage, $isRequired) {
        return new TCA_ActionBooleanSetParameter($name, $description, $usage, $isRequired);
    }
    
    static function newNumericSetParameter($name, $description, $usage, $isRequired, $min, $max) {
        return new TCA_ActionNumericSetParameter($name, $description, $usage, $isRequired, $min, $max);
    }
    
    static function newIntegerSetParameter($name, $description, $usage, $isRequired, $min, $max) {
        return new TCA_ActionIntegerSetParameter($name, $description, $usage, $isRequired, $min, $max);
    }
    
    static function newStringSetParameter($name, $description, $usage, $isRequired, $regExp, $regExpDescription) {
        return new TCA_ActionStringSetParameter($name, $description, $usage, $isRequired, $regExp, $regExpDescription);
    }
    
    static function newEnumeratedSetParameter($name, $description, $usage, $isRequired, TCA_Enumeration $enumeration) {
        return new TCA_ActionEnumeratedSetParameter($name, $description, $usage, $isRequired, $enumeration);
    }
}

?>
