<?php

require_once($_ENV['TCA_HOME']."/util/string.inc.php");
require_once($_ENV['TCA_HOME']."/cartridges/enumeration.inc.php");

interface TCA_ActionParameterValidationResultI {
    // returns boolean
    function isValid();
    
    // return error description if result is not valid
    function getErrorDescription();
}

//**************************************************************************
class TCA_ActionParameterValidationResult implements TCA_ActionParameterValidationResultI {
    private $isValid;
    private $errorDescription;
    
    function __construct($isValid, $errorDescription) {
    	$this->isValid = $isValid;
    	$this->errorDescription = $errorDescription;
    }
    
    function isValid() {
    	return $this->isValid;
    }
    
    function getErrorDescription() {
    	return $this->errorDescription;
    }
}

//**************************************************************************
interface TCA_ValidationRuleI {
    // returns boolean
    function isValid($value);
    
    // returns text description of the rule
    function getDescription();
}

class TCA_DummyValidationRule implements TCA_ValidationRuleI {

    private static $instance = null;

    private function __construct() {
    	// singleton
    }

    static function getInstance() {
    	if (self::$instance === null) {
    		self::$instance = new TCA_DummyValidationRule();
    	}
    	return self::$instance;
    }
    
    function isValid($value) {
    	return true;
    }
    
    function getDescription() {
    	return "Accept any value";
    }
}

//**************************************************************************
class TCA_BooleanValidationRule implements TCA_ValidationRuleI {
    function isValid($value) {
    	return is_numeric($value) && ($value == 0 || $value == 1) ? true : false;
    }
    
    function getDescription() {
    	return "Allowed values are 0 or 1";
    }
}

//**************************************************************************
class TCA_RegExpValidationRule implements TCA_ValidationRuleI {
    private $pattern;
    private $description;

    function __construct($pattern, $description) {
    	$this->pattern = $pattern;
    	$this->description = $description;
    }
    
    function isValid($value) {
    	return preg_match($this->pattern, $value) > 0 ? true : false;
    }
    
    function getDescription() {
    	return $this->description;
    }
    
    function getMatches($value) {
    	$matches = array();
    	preg_match($this->pattern, $value, $matches);
    	return $matches;
    }    
}

//**************************************************************************
class TCA_EnumeratedValidationRule implements TCA_ValidationRuleI {
    private $enumeration;
    private $description;
    
    public function __construct(TCA_Enumeration $enumeration) {
    	$this->enumeration = $enumeration;
    	$this->description = "Allowed values are: " . implode("|", $this->enumeration->getValues());
    }
    
    function getDescription() {
    	return $this->description;
    }
    
    function isValid($value) {
    	return $this->enumeration->getElement($value) != NULL ? true : false;
    }
    
    function getEnumeration() {
    	return $this->enumeration;
    }
}


//**************************************************************************

class TCA_NumericRangeValidationRule implements TCA_ValidationRuleI {
    protected $min; // inclusive
    protected $max; // exclusive
    
    function __construct($min, $max) {
    	$this->min = $min;
    	$this->max = $max;
    }

    function getDescription() {
    	return "Any number in [$this->min, $this->max) range";
    }
    
    function isValid($value) {
    	if (!is_numeric($value)) {
    		return false;
    	}
    	
    	return ($this->min <= $value && $value < $this->max) ? true : false;
    }
}

class TCA_IntegerRangeValidationRule extends TCA_NumericRangeValidationRule implements TCA_ValidationRuleI {
    function __construct($min, $max) {
    	parent::__construct(intval($min), intval($max));
    }

    function getDescription() {
    	return "Any integer in [$this->min, $this->max) range";
    }
    
    function isValid($value) {
    	if (!is_integer($value)) {
    		return false;
    	}
    	return parent::isValid($value);
    }
}


interface TCA_ActionParameterValidatorI {
    // returns TCA_ActionParameterValidationResultI
    function validate($value);
    
    // returns validation rule
    function getValidationRule();
}


//**************************************************************************
class TCA_GenericValidator implements TCA_ActionParameterValidatorI {
    private $rule;
    
    function __construct(TCA_ValidationRuleI $rule) {
    	$this->rule = $rule;
    }
	
    function getValidationRule() {
    	return $this->rule;
    }

    function validate($value) {
    	$errorDescription = "";
    	if (!$this->rule->isValid($value)) {
    		$errorDescription = $this->rule->getDescription();
    	}
    	
    	return new TCA_ActionParameterValidationResult($this->rule->isValid($value), $errorDescription);
    }
}

?>
