<?php

require_once("connector/connectable.inc.php");
require_once("connector/payload_translator.inc.php");
require_once("connector/cipher.inc.php");
require_once("connector/authenticationable.inc.php");
require_once("exceptions.inc.php");

// holder for connector request hash
class TCA_ConnectorRequest {
    private $request;

    function __construct() {
	$this->request = array();
    }

    function setRequest($req) {
	$this->request = $req;	
    }

    function getRequest() {
	return $this->request;
    }

    function __toString() {
	return TCA_String::toString($this);
    }
}

// holder for connector response hash
class TCA_ConnectorResponse {
    private $response;

    function setResponse($res) {
	$this->response = $res;
    }

    function getResponse() {
	return $this->response;
    }

    function __toString() {
	return TCA_String::toString($this);
    }
}

// Empty Shell used to pass auth objects
abstract class TCA_ResellerAuthInfo {
	
    // Method Used to return information requiried for cipher during encryption. 
    function getPrivateKey() { 
	return NULL;
    }
}

// Auth object for tucows services
class TCA_ResellerAuthInfoTucows extends TCA_ResellerAuthInfo {
    private $username;
    private $privateKey;

    function __construct($name, $key) {
	$this->setUsername($name);
	$this->setPrivateKey($key);
    }

    function setUsername($name) {
	$this->username = $name;
    }

    function getUsername() {
	return $this->username;
    }

    function setPrivateKey($key) {
	$this->privateKey = $key;
    }

    function getPrivateKey() {
	return $this->privateKey;
    }

    function __toString() {
	return "Username: [".$this->username.
		"], private key: [".$this->privateKey."]";
    }
}

// Auth object for blogware service
class TCA_ResellerAuthInfoBlogware extends TCA_ResellerAuthInfo {
    private $username;
    private $password;

    function __construct($name, $pass) {
	$this->setUsername($name);
	$this->setPassword($pass);
    }

    function setUsername($name) {
	$this->username = $name;
    }

    function getUsername() {
	return $this->username;
    }

    function setPassword($pass) {
	$this->password= $pass;
    }

    function getPassword() {
	return $this->password;
    }

    function __toString() {
	return "Username: [".$this->username.
		"], Password: [".$this->password."]";
    }
}

abstract class TCA_RemoteEnd {
	
}

class TCA_RemoteEndCBC extends TCA_RemoteEnd {
    private $host;
    private $port;

    function __construct($host, $port) {
	$this->setHost($host);
	$this->setPort($port);
    }

    function setHost($h) {
	$this->host = $h;
    }

    function getHost() {
	return $this->host;
    }

    function setPort($p) {
	$this->port = $p;
    }

    function getPort() {
	return $this->port;
    }

    function __toString() {
	return "Host: [".$this->host."], port: [".$this->port."]";
    }
}

class TCA_RemoteEndHTTPS extends TCA_RemoteEnd {
    private $host;
    private $port;

    function __construct($host, $port) {
	$this->setHost($host);
	$this->setPort($port);
    }

    function setHost($h) {
	$this->host = $h;
    }

    function getHost() {
	return $this->host;
    }

    function setPort($p) {
	$this->port = $p;
    }

    function getPort() {
	return $this->port;
    }

    function __toString() {
	return "Host: [".$this->host."], port: [".$this->port."]";
    }
}

class TCA_RemoteEndXMLRPC extends TCA_RemoteEnd {
    private $host;
    private $port;
    private $uri; // Specific to XMLRPC

    function __construct($host, $port, $uri) {
	$this->setHost($host);
	$this->setPort($port);
	$this->setURI($uri);
    }

    function setHost($h) {
	$this->host = $h;
    }

    function getHost() {
	return $this->host;
    }

    function setPort($p) {
	$this->port = $p;
    }

    function getPort() {
	return $this->port;
    }

    function setURI($u) {
	$this->uri = $u;
    }

    function getURI() {
	return $this->uri;
    }
   
    function __toString() {
	return "Host: [".$this->host."], port: [".$this->port."], URI: [".$this->uri."]";
    }
}

class TCA_Connector {
    // implementations, set in the constructor
    private $connectable;		// TCA_Connectable
    private $payloadTranslator;		// TCA_PayloadTranslatator
    private $cipher;			// TCA_Cipher
    private $authenticationable;	// TCA_Authenticationable
    private $logFile;			// FD used for loggin

    // resettable data 
    private $resellerAuthInfo;
    private $remoteEnd;

    // pass names of implementations 
    function __construct($connectableImplName, 
			 $payloadTranslatorImplName,
			 $cipherImplName) 
    {
	$this->setConnectable($connectableImplName);
	$this->setPayloadTranslator($payloadTranslatorImplName);
	$this->setCipher($cipherImplName);
	$this->setAuthenticationable();
    }

    function __destruct() {
	if($this->logFile)
		fclose($this->logFile);
    }

    function setConnectable($name) {
	$this->connectable = new $name;
	if (!$this->connectable instanceof TCA_Connectable) {
	    throw new TCA_IllegalArgumentsException();
	}
	if($this->logFile)
		$this->connectable->logFile = $this->logFile;
    }

    function getConnectable() {
    	return $this->connectable;
    }
    
    function setLogFileName($filename) {
	$this->logFile = fopen($filename,"a");
	if($this->logFile && $this->connectable)
	    $this->connectable->logFile = $this->logFile;
    }

    function setPayloadTranslator($name) {
	$this->payloadTranslator = new $name;
	if (!$this->payloadTranslator instanceof TCA_PayloadTranslator) {
	    throw new TCA_IllegalArgumentsException();
	}
    }

    function getPayloadTranslator() {
	return $this->payloadTranslator;    	
    }

    function setCipher($name) {
	$this->cipher = new $name;
	if (!$this->cipher instanceof TCA_Cipher) {
	    throw new TCA_IllegalArgumentsException();
	}
    }

    function getCipher() {
	return $this->cipher;    	
    }
    
    function setAuthenticationable() {    	
    	$class = $this->connectable->getAuthClass();
	$this->authenticationable = new $class($this);
	if (!$this->authenticationable instanceof TCA_Authenticationable) {
	    throw new TCA_IllegalArgumentException();
	}    	
    }

    function setResellerAuthInfo(TCA_ResellerAuthInfo $info) {
	$this->resellerAuthInfo = $info;
    }
    
    function getResellerAuthInfo() {
	return $this->resellerAuthInfo;
    }

    function setRemoteEnd(TCA_RemoteEnd $remote) {
	$this->remoteEnd = $remote;
    }

    function getRemoteEnd() {
	return $this->remoteEnd;
    }

    // This method accepts TCA_ConnectorRequest, packages it appropriately
    // and sends to server
    function sendCommand($req) {

	if ( !isset($this->resellerAuthInfo) ) {
	   throw new TCA_ObjectNotFullyInitializedException('resellerAuthInfo');
	}
	if ( !isset($this->remoteEnd) ) {
	    throw new TCA_ObjectNotFullyInitializedException('remoteEnd');
	}
	if ( !isset($this->authenticationable) ) {
	    throw new TCA_ObjectNotFullyInitializedException('authenticationable');
	}
	
	$this->cipher->initialize($this->resellerAuthInfo->getPrivateKey());
	
	// translate the payload, encrypt and send
	try {
	    $payload = $this->payloadTranslator->encode($req->getRequest(), $this->resellerAuthInfo);
    	$payload = $this->cipher->encrypt($payload);
	    $socket = $this->connectable->openSocket($this->remoteEnd, $this->resellerAuthInfo);
	    
	    if ( !$socket ) 
		throw new TCA_ConnectionException('Error opening socket');

	    if ( !$this->authenticationable->login() )
	        throw new TCA_ConnectionException('Authentication failed');	    
	    
	    $this->connectable->sendData($payload);
	    $response = $this->readResponse();
	    $connResp = new TCA_ConnectorResponse();
	    $connResp->setResponse($response);
	    return $connResp;
	} 
	catch ( TCA_ConnectionException $ConnectionException ) {
	    throw $ConnectionException;
	}	
	catch ( TCA_XMLParseException $XMLParseException ) {
	    throw $XMLParseException;
	}

    }

    function readResponse() {
	$response = $this->connectable->readData();
	$response = $this->cipher->decrypt($response);
	$response = $this->payloadTranslator->decode($response);
	return $response;
    }
}

?>
