<?php

interface TCA_Connectable {
    function openSocket(TCA_RemoteEnd $remote, TCA_ResellerAuthInfo $info);
    function closeSocket();
    function sendData($in);
    function readData();
    function getAuthClass();
}

class TCA_SocketConnector {
    protected $remoteEnd;
    protected $resellerAuthInfo;
    protected $socket;
    protected $authClass;
    public $logFile;
    
    function getAuthClass() {
	return $this->authClass;
    }
}

class TCA_CbcConnector extends TCA_SocketConnector implements TCA_Connectable {

    private $_CRLF;
    private $_timeout;
    
    function __construct() {
	$this->_CRLF = "\015\012";
	$this->_timeout = 5; //conf
	$this->authClass = "TCA_CbcAuthenticator";
    }
    
    function openSocket(TCA_RemoteEnd $remote, TCA_ResellerAuthInfo $info) {
	$this->remoteEnd = $remote;
	$this->resellerAuthInfo = $info;

	if ( !$this->socket ) {
	    $this->socket = fsockopen (
		$this->remoteEnd->getHost(), 
		$this->remoteEnd->getPort(), 
		$errno, $errstr, 30
	    );
	}
	
	return $this->socket;
    }

    function closeSocket() {
    	
    	if ( $this->socket ) {
	    fclose ($this->socket);
	    $this->socket = null;
	}
    }

    function sendData($in) {

	fputs($this->socket, 'Content-Length: ' . strlen($in) . 
	    $this->_CRLF . $this->_CRLF);
	fputs($this->socket, $in, strlen($in));

	if($this->logFile)
	    fwrite($this->logFile,"Sent:".$in."\r\n\r\n");
    }

    function readData() {

	$MAX_READ = 1024;
	$TRY = 10;
    	// set timeout on socket
    	stream_set_timeout($this->socket, $this->_timeout);
    
    	$resp = fgets($this->socket, 4096);
	$try_num = 0;
	while( !$resp && ++$try_num < 10 ) {
	    $resp = fgets($this->socket, 4096);
	}
    	// check the content length
        if ( preg_match('/^\s*Content-Length:\s+(\d+)\s*\r\n/i', $resp, $matches) ) {
	    $len = (int)$matches[1];
        } else {
	    throw new TCA_ConnectionException('Response missing content length');
        }
        
	if ( $this->socketStatus() ) {
	    // remove CRLF
	    $resp = fread($this->socket,2);
	    // read content
	    // somehow, if $len exceeds 1024 bytes, fread($this->socket, $len) does not work correctly
	    $lenLeft = $len;
	    while ($lenLeft > $MAX_READ) {
		$response .= fread($this->socket, $MAX_READ);
		$lenLeft -= $MAX_READ;
	    }
	    $response .= fread($this->socket, $lenLeft);
	} else {
	    throw new TCA_ConnectionException('Connection timeout');
	}

	if($this->logFile)
	    fwrite($this->logFile,"Recv:".$response."\r\n\r\n");

	return $response;
    }
        
    private function socketStatus() {

    	$retval = true;
	if ( is_resource($this->socket) ) {
	    $temp = socket_get_status($this->socket);
	    if ( $temp['timed_out'] ) {
	    	$retval = false;
	    }
	    unset($temp);
	}	
	return $retval;
    }

}

class TCA_HttpsConnector extends TCA_SocketConnector implements TCA_Connectable {
    
    private $_CRLF;

    function __construct() {
	$this->_CRLF = "\015\012";
	$this->authClass = "TCA_HttpsAuthenticator";
    }

    function __destruct() {
	$this->closeSocket();
    }
    
    function openSocket(TCA_RemoteEnd $remote, TCA_ResellerAuthInfo $info) {

    	$this->remoteEnd = $remote;
	$this->resellerAuthInfo = $info;
    	
    	$host = $this->remoteEnd->getHost();
	$port = $this->remoteEnd->getPort();
	     	
	if ( !$this->socket )
	    $this->socket = fsockopen ("ssl://$host", $port, $errno, $errstr, 30);
	
	return $this->socket;
    }
    
    function closeSocket() {

    	if ( $this->socket ) {
	    fclose ($this->socket);
	    $this->socket = null;
	}
    }
    
    function sendData($in) {

    	$signature = md5(md5($in.$this->resellerAuthInfo->getPrivateKey()) . 
	    $this->resellerAuthInfo->getPrivateKey());
	$url = "/"; 
	
	$request .= "POST $url HTTP/1.0" . $this->_CRLF; 
	$request .= "Content-Type: text/xml" . $this->_CRLF;
	$request .= "X-Username: " . $this->resellerAuthInfo->getUsername() . 
	    $this->_CRLF;
	$request .= "X-Signature: $signature" . $this->_CRLF; 
	$request .= "Keep-Alive: off" . $this->_CRLF; 
	$request .= "Content-Length: " . strlen($in) . 
	    $this->_CRLF . $this->_CRLF;
	
	fputs ($this->socket, $request . $in);
	
	if($this->logFile)
	    fwrite($this->logFile,"Sent:".$request.$in."\r\n\r\n");
    }
    
    function readData() {
    	
	while ( !feof($this->socket) ) { 
	    $resp = fgets ($this->socket, 1024);
	    if ( $resp == $this->_CRLF ) {
		$response = '';
	    } else {
		$response .= $resp;
	    }
	}
	$this->closeSocket();

	if($this->logFile)
	    fwrite($this->logFile,"Recv:".$response."\r\n\r\n");

	return $response;
    }
}

class TCA_XMLRPCConnector extends TCA_SocketConnector implements TCA_Connectable {
    
    private $_CRLF;

    function __construct() {
	$this->_CRLF = "\015\012";
	$this->authClass = "TCA_HttpsAuthenticator";
    }

    function __destruct() {
	$this->closeSocket();
    }
    
    function openSocket(TCA_RemoteEnd $remote, TCA_ResellerAuthInfo $info) {

    $this->remoteEnd = $remote;
	$this->resellerAuthInfo = $info;
    	
    $host = $this->remoteEnd->getHost();
	$port = $this->remoteEnd->getPort();
	     	
	if ( !$this->socket )
	    $this->socket = fsockopen ("ssl://$host", $port, $errno, $errstr, 30);
	
	return $this->socket;
    }
    
    function closeSocket() {

    	if ( $this->socket ) {
	    fclose ($this->socket);
	    $this->socket = null;
	}
    }
    
    function sendData($in) {

    $url = $this->remoteEnd->getURI(); 
    	$request = '';
	
	$request .= "POST $url HTTP/1.0" . $this->_CRLF; 
	$request .= "Content-Type: text/xml" . $this->_CRLF;
	$request .= "Keep-Alive: off" . $this->_CRLF; 
	$request .= "Content-Length: " . strlen($in) . 
	    $this->_CRLF . $this->_CRLF;
	
	fputs ($this->socket, $request . $in);
	
	if($this->logFile) {
	    fwrite($this->logFile,"Sent:".$request.$in."\r\n\r\n");
	}
    }
    
    function readData() {
    	
	while ( !feof($this->socket) ) { 
	    $resp = fgets ($this->socket, 1024);
	    if ( $resp == $this->_CRLF ) {
		$response = '';
	    } else {
		$response .= $resp;
	    }
	}
	$this->closeSocket();
	
	if($this->logFile)
	    fwrite($this->logFile,"Recv:".$response."\r\n\r\n");

	return $response;
    }
}

?>
