<?php

require_once("util/parameter.inc.php");

class TCA_ServiceRequestParameter extends TCA_Parameter {}

class TCA_ServiceRequestParameterSet extends TCA_ParameterSet {

    public static function createServiceRequestParameterSet(TCA_ParameterSetI $params) {
	$pSet = new TCA_ServiceRequestParameterSet();
	foreach ($params->keys() as $key) {
	    $param = $params->getElement($key);
	    if ($param instanceof TCA_ParameterSetI) {
		$pSet->addParameterSet($key, $param);
	    } else if ($param instanceof TCA_ParameterI) {
		$pSet->addParameter($param);
	    } else {
		throw new Exception("can't go here!!!");
	    }
	}
	return $pSet;
    }
}

abstract class TCA_ServiceValidator {
    protected $genericRuleList;
    protected $validator;

    abstract function initGenericValidationRuleList();
    abstract function validateServiceRequest(
	TCA_ValidationRuleList $ruleList, TCA_ServiceRequest $reqest);

    final function getGenericValidationRuleList() {
	return $this->genericRuleList;
    }
}

abstract class TCA_ValidationMapper {
    protected $map;
    
    abstract function initValidationMap();
        
    final function getRuleList($key) {
	if ( !isset($this->map[$key]) )
	    throw new Exception("Action [$key] does not exist.");
	return $this->map[$key];
    }
}

abstract class TCA_ActionMapper {
    protected $actionList;
    
    final function getAction($action) {
	return $this->actionList->getElement($action);
    }
    
    final function setAction($action, TCA_ApiAction $apiAction) {
	$this->actionList->addApiAction($action, $apiAction);
    }
}

class TCA_ApiAction {
    protected $simpleParams;
    protected $expectedParams;
	
    function __construct($params) {
    	$this->simpleParams = $params{'simple'};
    	$this->expectedParams = $params{'expected'};
    }

    function getSimpleParams() {
	return $this->simpleParams;
    }
    
    function getExpectedParams() {
	return $this->expectedParams;
    }
}

class TCA_ApiActionList {
    private $actionList = array();

    static function createApiActionList($ar) {
	if ( !is_array($ar)) throw new Exception("Parameter must be array");

	$list = new TCA_ApiActionList();
	foreach ( $ar as $key => $element) {
	    if ( $element instanceof TCA_ApiActionList ) {
		$list->addApiActionList($key, $element);
	    } else if ( $element instanceof TCA_ApiAction ) {
		$list->addApiAction($key, $element);
	    } else {
		throw new Exception("Wrong element type");
	    }
	}

	return $list;
    }

    function addApiAction($key, TCA_ApiAction $apiAction) {
	$this->actionList[$key] = $apiAction;
    }

    function addApiActionList($key, TCA_ApiActionList $list) {
	if (isset($this->actionList[$key]))
	    throw new Exception("Action list element [$key] already defined");
    
	$this->actionList[$key] = $list;
    }

    function getElement($key) {
    	return $this->actionList[$key];
    }

    function keys() {
	return array_keys($this->actionList);
    }
}

class TCA_WorkOrderConverter {

    function getWorkOrder(TCA_ApiAction $apiAction, TCA_ServiceRequest $request) {
	$workOrder = new TCA_WorkOrder();
	$workUnitBundle = new TCA_WorkUnitBundle();
	$workUnit = new TCA_WorkUnit();

	foreach ( $apiAction->getSimpleParams() as $key => $value ) {
	    $workUnit->addParameter(new TCA_WorkUnitParameter($key, $value));
	}
	
	$reqArr = $request->toArray();
	foreach ( $reqArr as $key => $value ) {
	    if ( is_array($value) ) {
		$pSet = new TCA_WorkUnitParameterSet();
		$pSet = $pSet->fromArray($value);	    
		$workUnit->addParameterSet($key, $pSet);
	    } else {
	    	$workUnit->addParameter(new TCA_WorkUnitParameter($key, $value));
	    }		
	}	

	$workUnitBundle->addWorkUnit($workUnit);
	$workOrder->addWorkUnitBundle($workUnitBundle);
	return $workOrder;
    }
}

class TCA_ServiceRequest {
    private $action = "";
    private $parameters;
    private $provider = "";

    function __construct($action) {
	$this->action = $action;
	$this->parameters = new TCA_ServiceRequestParameterSet();
    }

    function getAction() {
	return $this->action;
    }

    function getProvider() {
	return $this->provider;
    }

    function setProvider($provider) {
    	$this->provider = $provider;
    }

    function getActionPrefix() {
	return substr($this->action, 0, strpos($this->action, '.'));
    }

    function getActionPostfix() {
	if(strpos($this->action,'.'))
	    return substr($this->action, strpos($this->action, '.') + 1);
    	else 
	    return $this->action;
    }

    function addParameter(TCA_ServiceRequestParameter $par) {
	$this->parameters->addParameter($par);
    }
    
    function addParameterSet($key, TCA_ServiceRequestParameterSet $set) {
	$this->parameters->addParameterSet($key, $set);
    }

    function getElement($key) {
	return $this->parameters->getElement($key);
    }

    function __toString() {
	$str = "TCA_ServiceRequest, type: [$this->action], parameters:\n";
	$str .= $this->parameters->__toString() . "\n";
	return $str;
    }

    function toArray() {
	$a = array();
	foreach ( $this->parameters->keys() as $key) {
	    $par = $this->parameters->getElement($key);
	    if ( $par instanceof TCA_ServiceRequestParameterSet ) {
		$a[$key] = $par->toArray();
	    } else if  ( $par instanceof TCA_ServiceRequestParameter) {
		$a[$key] = $par->getValue();
	    } else {
		throw new Exception("Wrong element type");
	    }
	}

	return $a;
    }
}

abstract class TCA_ServiceResponseStatus {
    protected $isSuccess = 0;
    protected $responseCode;
    protected $responseText;

    abstract function isSuccess();
}

class TCA_ServiceResponse extends TCA_ServiceResponseStatus {
 
    private $parameterSet;

    function __construct() {
	$this->parameterSet = new TCA_ServiceRequestParameterSet();
    }

    function setSuccess($success) {
	$this->isSuccess = $success == 1 ? 1 : 0;
    }

    function isSuccess() {  
	return $this->isSuccess;
    }

    function setResponseCode($code) {
	$this->responseCode = $code;
    }

    function getResponseCode() {
	return $this->responseCode;
    }

    function setResponseText($message) {
	$this->responseText = $message;
    }

    function getResponseText() {
	return $this->responseText;
    }

    function getResponseData() {
	return $this->parameterSet;
    }
        
    function fromWorkOrderResponse($workOrderResponse) {
	$this->isSuccess = $workOrderResponse->isSuccess();
	$this->responseCode = $workOrderResponse->getResponseCode();
	$this->responseText = $workOrderResponse->getResponseText();
	$this->parameterSet = $workOrderResponse->getResponseData();	
    }

    function __toString() {
	return "TCA_ServiceResponse, ".$this->parameterSet->__toString()."\n";
    }
}

class TCA_ServiceController {
    private $workOrderManager;
    private $cartridgeManager;
    private $serviceManagers = array();
    private $actionPrefixes = array();

    function setWorkOrderManager(TCA_WorkOrderManager $woManager) {
	$this->workOrderManager = $woManager;	
    }

    function setCartridgeManager(TCA_CartridgeManager $crtManager) {
	$this->cartridgeManager = $crtManager;
    }

    function init() {
	$cartridges = $this->cartridgeManager->getCartridges();
	foreach ( $cartridges as $cartridge ) {	
	    $this->registerServiceManager($cartridge->getServiceManager());
	}
    }

    function registerServiceManager($serviceManager) {
	$manager = $serviceManager;
	$manager->setServiceController($this);
	$this->serviceManagers[$serviceManager->getName()] = $manager;

	// define the SM name for each service action
	$actionPrefix = $manager->getActionPrefix();

	// checking for duplicate action prefix
	if ( $this->actionPrefixes[$actionPrefix]) {
	    throw new Exception("Service manager with duplicate action ".
				"prefix: [$actionPrefix]");
	}

	# service controller only knows prefixes now
	$this->actionPrefixes[$actionPrefix] = $manager;
    }

    function getServiceManager($clazz) {
	return $this->serviceManagers[$clazz];
    }

    function getServiceManagerByActionPrefix($actionPrefix) {
	return $this->actionPrefixes[$actionPrefix];
    }

    # action names now contain prefix, e.g. 'domain.lookup_domain', so 
    # delegate to SM
    function isValidAction( TCA_ServiceRequest $req ) {
	$prefix = $req->getActionPrefix();
	$postfix = $req->getActionPostfix();
	if ( strlen($prefix) == 0 || strlen($postfix) == 0) {
	    return 0;
	}

	$man = $this->getServiceManagerByActionPrefix($prefix);
	if ( !$man ) return 0;
	
	return ($man->isValidAction($postfix) ? 1 : 0);
    }

    function processServiceRequest( TCA_ServiceRequest $req) {
	// find which SM does this action
	if ( !$this->isValidAction($req) ) {
	    throw new TCA_InvalidServiceActionException($req->getAction());
	}

	$man = $this->getServiceManagerByActionPrefix($req->getActionPrefix());
	$man->setWorkOrderManager($this->workOrderManager);
	return $man->processServiceRequest($req);
    }
}

abstract class TCA_AbstractServiceManager {
    private $serviceController;
    private $workOrderManager;
    protected $actionMapper;
    protected $actionValidationMapper;

    abstract function init();
    abstract function getActionPrefix();
    abstract function getActions();
    abstract function getRequestValidator();

    function __construct() {}

    final function getName() {
	return get_class($this);
    }
    
    function setWorkOrderManager(TCA_WorkOrderManager $woManager) {
	$this->workOrderManager = $woManager;    	
    }
    
     function getWorkOrderManager() {
	return $this->workOrderManager ;    	
    }
    
    final function setServiceController(TCA_ServiceController $controller) {
	$this->serviceController = $controller;
    }

    final function isValidAction($action) {	
	$actions = $this->getActions();
	return ($actions[$action] == 1 ? 1 : 0);
    }

    final function createServiceRequest($action) {
	if ( ! $this->isValidAction($action)) {
	    throw new TCA_InvalidServiceActionException($action);
	}
	return new TCA_ServiceRequest($action);
    }

    // dispatcher to action methods in subclass
    final function processServiceRequest(TCA_ServiceRequest $request) {
	$action = $request->getActionPostfix();
	if (!$this->isValidAction($action)) {
	    throw new TCA_InvalidServiceActionException($action);
	}

	try {
	    $this->validateServiceRequest($request);
	    // dispatch to the right method
	    $method = $request->getActionPostfix();
	    $serviceResponse = $this->$method($request);
	} catch ( TCA_InvalidServiceRequestException $e) {
	    throw $e;
	}
	return $serviceResponse;
    }

    final function validateServiceRequest(TCA_ServiceRequest $req) {		
	// first, generic level of checking
	$this->getRequestValidator()->validateServiceRequest(
	    $this->getRequestValidator()->getGenericValidationRuleList(),$req);
	// second, action specific level of checking
	$this->getRequestValidator()->validateServiceRequest(
	    $this->actionValidationMapper->getRuleList($req->getActionPostfix()),$req);	
    }
    
    function processWorkOrder(TCA_WorkOrder $workOrder, $provider) {
	$serviceResponseArr = array();
	
    	$this->workOrderManager->setWorkOrder($workOrder);
	$workOrderResponse = $this->workOrderManager->process($provider);	
	
	foreach ( $workOrderResponse->getWorkUnitBundleResponseList() as $workUnitBunldeResponse ) {
	    foreach ( $workUnitBunldeResponse->getWorkUnitResponseList() as $workUnitResponse ) {
		$serviceResponse = new TCA_ServiceResponse();
	        $serviceResponse->fromWorkOrderResponse($workUnitResponse);
	        array_push($serviceResponseArr, $serviceResponse);
	    }
	}
	return $serviceResponseArr;
    }

    function composeWorkOrder($action, TCA_ServiceRequest $request) {
    	$converter = new TCA_WorkOrderConverter();
    	return $converter->getWorkOrder($this->actionMapper->getAction($action),$request);
    }
    
    function processOrder($action, TCA_ServiceRequest $request) {
    	$workOrder = $this->composeWorkOrder($action, $request);   
	return $this->processWorkOrder($workOrder, $request->getProvider());	
    }
}

?>
