<?php

require_once($_ENV['TCA_HOME']."/util/string.inc.php");

// Class framework for data validation. See usage/validator.php for usages.

// Helper class for more precise error reporting
class TCA_ErrorLocator {
    private $locator = array();
    private $level = 0;

    function clear() {
        $this->locator = array();
    }

    function incLevel() {
        $this->level++;
    }

    function decLevel() {
        $this->level--;
    }

    function isEmpty() {
        return empty($this->locator);
    }

    function setKey($key) {
        $this->locator[$this->level] = $key;
    }

    function __toString() {
        return implode(', ', $this->locator);
    }
}

class TCA_ValidationErrorMsg {
    private $msg;
    private $locatorKey;

    function __construct($msg, $locatorKey) {
        $this->msg = $msg;
        $this->locatorKey = $locatorKey;
    }

    function getMsg() {
        return $this->msg;
    }

    function getLocatorKey() {
        return $this->locatorKey;
    }
}

class TCA_ValidationResult {
    private $isValid = 0;
    private $errorMsg = array();

    function setValid($valid) {
        $this->isValid = $valid;
    }

    function isValid() {
        return $this->isValid;
    }

    function addLocatedErrorMsg($locatorKey, $msg) {
        array_push($this->errorMsg,
                new TCA_ValidationErrorMsg($msg, $locatorKey)
        );
    }

    function addErrorMsg($msg) {
        array_push($this->errorMsg,
                new TCA_ValidationErrorMsg($msg, '')
        );
    }

    function getLastError() {
        return end($this->errorMsg)->getMsg();
    }

    function getLastLocatorKey() {
        return end($this->errorMsg)->getLocatorKey();
    }

    function getAllErrorMsgs() {
        return $this->errorMsg;
    }

    function __toString() {
        $str = "Validation result: ".($this->isValid ? "OK" : "Error")."\n";
        foreach ($this->errorMsg as $msg) {
            if ($msg->getLocatorKey())
                $str .= "Locator: [".$msg->getLocatorKey()."]. ";
            $str .= "Message: [".$msg->getMsg()."]\n";
        }
        return $str;
    }
}

class TCA_Validator {
    private $res;
    private $locator;
    private $optData;

    function __construct() {
        $this->res = new TCA_ValidationResult();
        $this->res->setValid(1);
        $this->locator = new TCA_ErrorLocator();
        unset($this->optData);
    }

    function clearErrors() {
        $this->res = new TCA_ValidationResult();
        $this->res->setValid(1);
        $this->locator->clear();
        unset($this->optData);
    }

    function setOptData($arg) {
        $this->optData = $arg;
    }

    function getOptData() {
        return $this->optData;
    }

    function validate() {
        $this->clearErrors();

        $arg_list = func_get_args();
        $rules = $arg_list[0];
        $ar = $arg_list[1];

        if ($arg_list[2]) {
            $this->setOptData($arg_list[2]);
        }
        return $this->validateRuleList($rules, $ar);
    }

    // recursive walk into RuleList
    function validateRuleList(TCA_ValidationRuleList $rules, $ar) {
        foreach ($rules->keys() as $key) {
            $val = $rules->getElement($key);

            // found another RuleList, call itself recursively
            if ($val instanceof TCA_ValidationRuleList) {
                $this->locator->setKey($key);
                $this->locator->incLevel();
                $this->validateRuleList($val, $ar[$key]);
                $this->locator->decLevel();

            // found the rule array, iterate through all rules in sequence
            } elseif (is_array($val)) {
            // if the array key is 'int', assume it's a list of rules
            // for the same data item
                if (is_int($key))
                    $data = $ar;
                else {
                    $data = $ar[$key];
                    $this->locator->setKey($key);
                }

                foreach ($val as $rule) {
                    if (!isset($data)) {
                        if ($rule->isRequired()) {
                            $this->res->setValid(0);
                            $this->res->addLocatedErrorMsg(
                                    $this->locator->__toString(),
                                    "Required field not present."
                            );
                        }
                        continue;
                    }

                    // validate individual rule
                    if (isset($this->optData)) {
                        $rule->setOptData($this->getOptData());
                    }
                    $res = $rule->validate($data);
                    if (!$res->isValid()) {
                        $this->res->setValid(0);
                        $this->res->addLocatedErrorMsg(
                                $this->locator->__toString(),
                                $res->getLastError()
                        );
                    }
                }

                // shouldn't ever come here
            } else {
                throw new Exception("Wrong element type");
            }
        }
        return $this->res;
    }
}

// This class allows to build multi-level hierarchical rule structures:
// the leaf nodes are individual rules and all intermediaries are rule lists.
// The idea is to define these structures to follow the structures of service
// requests (or any other multiple level array trees) to make it easy to
// generically validate initialized service requests objects.
class TCA_ValidationRuleList {
    private $ruleList = array();

    // so we can't overload constructors in PHP, - use factory methods instead
    static function createRuleList($ar) {
        if (!is_array($ar))
            throw new Exception("Parameter must be array");

        $list = new TCA_ValidationRuleList();
        foreach ($ar as $key => $element) {
            if ($element instanceof TCA_ValidationRuleList) {
                $list->addRuleList($key, $element);
            } elseif ($element instanceof TCA_ValidationRule) {
                $list->addRule($key, $element);
            } else {
                throw new Exception("Wrong element type");
            }
        }
        return $list;
    }

    function addRule($key, TCA_ValidationRule $rule) {
        if (!isset($this->ruleList[$key]))
            $this->ruleList[$key] = array();
        array_push($this->ruleList[$key], $rule);
    }

    function addRuleList($key, TCA_ValidationRuleList $list) {
        if (isset($this->ruleList[$key]))
            throw new Exception("Rule list element [$key] already defined");
        $this->ruleList[$key] = $list;
    }

    function setRuleList($key, TCA_ValidationRuleList $list) {
        $this->ruleList[$key] = $list;
    }

    function getElement($key) {
        return $this->ruleList[$key];
    }

    function keys() {
        return array_keys($this->ruleList);
    }
}

abstract class TCA_ValidationRule {
    private $isRequired = 0;
    private $optData;

    function isRequired() {
        return $this->isRequired;
    }

    function setRequired($val) {
        $this->isRequired = $val;
    }

    function getOptData() {
        return $this->optData;
    }

    function setOptData($opt) {
        $this->optData = $opt;
    }

    abstract function validate($data);
}

class TCA_EnumValidationRule extends TCA_ValidationRule {
    private $valueSet = array();

    static function createRule($required, $ar) {
        if (!is_array($ar))
            throw new Exception("Parameter must be array");

        $rule = new TCA_EnumValidationRule();
        $rule->setRequired($required);
        $rule->setValues($ar);
        return $rule;
    }

    function setValues($list) {
        foreach ($list as $el) {
            $this->valueSet[$el] = 1;
        }
    }

    function addValue($val) {
        $this->valueSet[$val] = 1;
    }

    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if (!$this->valueSet[$data]) {
            $res->setValid(0);
            $res->addErrorMsg(
                    "Value [$data] is not in the list of valid ".
                    "values: ".
                    implode(", ", array_keys($this->valueSet))
            );
        }
        return $res;
    }
}

class TCA_SubsetValidationRule extends TCA_ValidationRule {
    private $valueSet = array();

    static function createRule($required, $ar) {
        if (!is_array($ar))
            throw new Exception("Parameter must be array");

        $rule = new TCA_SubsetValidationRule();
        $rule->setRequired($required);
        $rule->setValues($ar);
        return $rule;
    }

    function setValues($list) {
        $this->valueSet = $list;
    }

    function addValue($val) {
        if (is_array($val)) {
            $this->valueSet = $this->mergeArray($this->valueSet, $val);
        } else {
            $this->valueSet = $this->mergeArray($this->valueSet, array($val));
        }
    }

    private function mergeArray($ar1, $ar2) {
        foreach (array_values($ar2) as $val) {
            if (array_search($val, $ar1) === false) {
                array_push($ar1, $val);
            }
        }
        return $ar1;
    }

    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);
        if (! is_array($data)) {
            $res->setValid(0);
            $res->addErrorMsg("Value is not an array");
        } elseif ($this->mergeArray($this->valueSet, $data) !== $this->valueSet) {
            $res->setValid(0);
            $res->addErrorMsg(
                    "Value is not in the list of valid ".
                    "values: ".
                    implode(", ", array_values($this->valueSet)));
        }
        return $res;
    }
}

class TCA_MinArrayLengthValidationRule extends TCA_ValidationRule {
    private $min;

    static function createRule($required, $min) {
        $rule = new TCA_MinArrayLengthValidationRule();
        $rule->setRequired($required);
        $rule->setMinLimit($min);
        return $rule;
    }

    function setMinLimit($min) {
        $this->min = $min;
    }

    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if (!is_array($data)) {
            $res->setValid(0);
            $res->addErrorMsg("Value is not an array");
        }
        foreach ($data as $key => $value) {
            if (!is_int($key)) {
                $res->setValid(0);
                $res->addErrorMsg("Value has invalid array key");
            }
        }
        if (count($data) < $this->min) {
            $res->setValid(0);
            $res->addErrorMsg("Size of Array below minimum: [$this->min]");
        }
        return $res;
    }
}

// TCA_NullValidationRule is the most primitive rule type.
// If 'required' is not set, it will always return valid;
// if 'requied' is set, it will return valid if the field is present,
// and value is not empty.
class TCA_NullValidationRule extends TCA_ValidationRule {
    static function createRule($required) {
        $rule = new TCA_NullValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if ($this->isRequired() && isset($data) && !is_array($data) &&
                (preg_match('/^\s+$/', $data) || $data == '')
        ) {
            $res->setValid(0);
            $res->addErrorMsg("Value must not be empty.");
        }
        return $res;
    }
}

class TCA_MinValidationRule extends TCA_ValidationRule {
    private $min;

    static function createRule($required, $min) {
        $rule = new TCA_MinValidationRule();
        $rule->setRequired($required);
        $rule->setMinLimit($min);
        return $rule;
    }

    function setMinLimit($min) {
        $this->min = $min;
    }

    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if (isset($this->min) && $data < $this->min) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$data] below minimum: [$this->min]");
        }
        return $res;
    }
}

class TCA_MaxValidationRule extends TCA_ValidationRule {
    private $max;

    static function createRule($required, $max) {
        $rule = new TCA_MaxValidationRule();
        $rule->setRequired($required);
        $rule->setMaxLimit($max);
        return $rule;
    }

    function setMaxLimit($max) {
        $this->max = $max;
    }

    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if (isset($this->max) && $data > $this->max) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$data] above maximum: [$this->max]");
        }
        return $res;
    }
}

class TCA_MinLengthValidationRule extends TCA_ValidationRule {
    private $minLength;

    static function createRule($required, $minLength) {
        $rule = new TCA_MinLengthValidationRule();
        $rule->setRequired($required);
        $rule->setMinLength($minLength);
        return $rule;
    }

    function setMinLength($min) {
        $this->minLength = $min;
    }

    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if (isset($this->minLength) && strlen($data) < $this->minLength) {
            $res->setValid(0);
            $res->addErrorMsg(
                    "Length of string [$data] below minimum length: ".
                    "[$this->minLength]"
            );
        }
        return $res;
    }
}

class TCA_MaxLengthValidationRule extends TCA_ValidationRule {
    private $maxLength;

    static function createRule($required, $maxLength) {
        $rule = new TCA_MaxLengthValidationRule();
        $rule->setRequired($required);
        $rule->setMaxLength($maxLength);
        return $rule;
    }

    function setMaxLength($max) {
        $this->maxLength = $max;
    }

    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if (isset($this->maxLength) && strlen($data) > $this->maxLength) {
            $res->setValid(0);
            $res->addErrorMsg(
                    "Length of string [$data] above maximum length: ".
                    "[$this->maxLength]"
            );
        }
        return $res;
    }
}

class TCA_RegexValidationRule extends TCA_ValidationRule {
    private $regex;

    static function createRule($required, $regex) {
        $rule = new TCA_RegexValidationRule();
        $rule->setRequired($required);
        $rule->setRegex($regex);
        return $rule;
    }

    function setRegex($regex) {
        $this->regex = $regex;
    }

    final function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if (!preg_match($this->regex, $data)) {
            $res->setValid(0);
            $res->addErrorMsg(
                    "Value [$data] does not pass regex test: ".
                    "[$this->regex]"
            );
        }
        return $res;
    }
}

class TCA_IPAddressValidationRule extends TCA_RegexValidationRule {
    static function createRule($required, $regex = false) {
        $rule = new TCA_IPAddressValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    function __construct() {
        $ReIpNum = "([01]?\d\d?|2[0-4]\d|25[0-5])";
        $this->setRegex("/^$ReIpNum\.$ReIpNum\.$ReIpNum\.$ReIpNum$/");
    }
}

class TCA_DomainSyntaxValidationRule extends TCA_ValidationRule {
    static function createRule($required) {
        $rule = new TCA_DomainSyntaxValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    final function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if (!preg_match('/^[a-z0-9\.-]+$/', $data) ||
            !preg_match('/(?:[a-z0-9](?:[-a-z0-9]*[a-z0-9])*\.)+[a-z]{2,}/', $data)) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$data] is not a valid domain name");
        }
        return $res;
    }
}

class TCA_HostnameValidationRule extends TCA_RegexValidationRule {
    static function createRule($required, $regex = false) {
        $rule = new TCA_HostnameValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    function __construct() {
        $this->setRegex("/(?:[a-z0-9](?:[-a-z0-9]*[a-z0-9])*\.{0,1})+/");
    }
}

class TCA_DomainPasswordValidationRule extends TCA_ValidationRule {
    private $regex;

    function __construct() {
        $this->regex = "/^[A-Za-z0-9\[\]\(\)!@\$\^,\.~\|=\-\+_\{\}\#]+$/";
    }

    static function createRule($required) {
        $rule = new TCA_DomainPasswordValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    function setRegex($regex) {
        $this->regex = $regex;
    }


    function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        if ((!trim($data)) || (strlen($data) < 3) || (strlen($data) > 20)) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$data] does not pass length test: password must be between 3 and 20 characters");
            return $res;
        }

        if (!preg_match($this->regex, $data)) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$data] does not pass regex test: " . "[$this->regex]");
        }
        return $res;
    }
}

// Email addresses can include whitespace and nested comments and so regex
// can not possibly validate all different kinds of email addresses.

// Update: validator now looks for <>.  If they exist, only the contents
// of the brackets are validated.
class TCA_EmailAddressValidationRule extends TCA_ValidationRule {
    static function createRule($required) {
        $rule = new TCA_EmailAddressValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    final function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        $strEmail = $data;
        $intStart = strpos($strEmail, '<');
        $intEnd = 0;
        if ($intStart !== false) {
            $intStart += 1;
            $strEmail = substr($strEmail, $intStart);
            $intEnd = strpos($strEmail, '>');
            if ($intEnd !== false)
                $strEmail = substr($strEmail, 0, $intEnd-1);
        } else {
            $intStart = strpos($strEmail, 'lt;');
            if ($intStart !== false) {
                $intStart += 3;
                $strEmail = substr($strEmail, $intStart);
                $intEnd = strpos($strEmail, 'gt;');
                if ($intEnd !== false)
                    $strEmail = substr($strEmail, 0, $intEnd-1);
            }
        }

        if (!preg_match("/^[_A-Za-z0-9-]+(\.[_A-Za-z0-9-]+)*@[A-Za-z0-9-]+(\.[A-Za-z0-9-]+)+$/", $strEmail)) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$data] is not a valid email address");
        }

        return $res;
    }
}

class TCA_PhoneValidationRule extends TCA_ValidationRule {
    static function createRule($required) {
        $rule = new TCA_PhoneValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    final function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        $phone = preg_replace("/ +/", "", $data);
        if ((strlen($data) > 20) ||
                ((!preg_match('/^\+\d{1,3}\.\d{1,12}(x\d{1,6})?$/', $phone)) &&
                (!preg_match('/^[\d\-\(\)\.\+\#\*]{4,}((x|ex|ext|xt)\.?\d+)?(TDD)?$/i', $phone))) ||
                (preg_match('/\(/', $phone) != preg_match('/\)/', $phone))
        ) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$data] is not a valid phone number.");
        }
        return $res;
    }
}

class TCA_TransferListValidationRule extends TCA_ValidationRule {

    static function createRule($required) {
        $rule = new TCA_TransferListValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    final function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        // Check for array
        if (!is_array($data) || !$data) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$data] is not a valid Domain List.");
            return $res;
        }
        $domainRule = new TCA_DomainSyntaxValidationRule();
        $domaintld = "";

        // Check for vaild domain names and com/net
        foreach ($data as $domain) {
            // Check for valid domain
            $dres = $domainRule->validate($domain);
            if (!$dres->isValid()) {
                $res->setValid(0);
                $res->addErrorMsg("$domain is not a valid domain name.");
                break;
            }

            if (!$domaintld)
                $domaintld = strrev(substr(strrev($domain),0,strpos(strrev($domain),".")));

            // Check for com/net
            if ((strcasecmp($domaintld,"com") == 0) || (strcasecmp($domaintld,"net") == 0)) { // special Case for com/net
                if (!preg_match('/.(com|net)$/', $domain)) {
                    $res->setValid(0);
                    $res->addErrorMsg("This list is invalid, because it contains types of domains that cannot be combined. If you want to transfer several domains at once, please be aware that only:\n1. domains of the same type\n2. '.com' and '.net' domains\ncan be combined within the same order. All other combinations of the TLDs are unsupported for batch transfer purposes.\n$domain is not a .com or .net domain.");
                    break;
                }
            } else {
                if (!preg_match("/.(".$domaintld.")$/", $domain)) {
                    $res->setValid(0);
                    $res->addErrorMsg("This list is invalid, because it contains types of domains that cannot be combined. If you want to transfer several domains at once, please be aware that only:\n1. domains of the same type\n2. '.com' and '.net' domains\ncan be combined within the same order. All other combinations of the TLDs are unsupported for batch transfer purposes.");
                    break;
                }
            }
        }
        return $res;
    }
}

class TCA_PhoneValidationRuleEPP extends TCA_ValidationRule {

    static function createRule($required) {
        $rule = new TCA_PhoneValidationRuleEPP();
        $rule->setRequired($required);
        return $rule;
    }

    final function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        $phone = preg_replace("/ +/", "", $data);
        if ((strlen($data) > 20) ||
                (!preg_match('/^\+\d{1,3}\.\d{1,12}(x\d{1,6})?$/', $phone))
        ) {
            $res->setValid(0);
            $res->addErrorMsg(
                    "Value [$data] is not a valid phone number. ".
                    "Phone number has to be in +1.4165551122x1234 format."
            );
        }
        return $res;
    }
}

// Custom function must return 1 or 0.
class TCA_CustomValidationRule extends TCA_ValidationRule {
    private $argsDef;
    private $funcDef;

    static function createRule($required, $args, $func) {
        $rule = new TCA_CustomValidationRule();
        $rule->setRequired($required);
        $rule->setFunction($args, $func);
        return $rule;
    }

    function setFunction($args, $func) {
        $this->argsDef = $args;
        $this->funcDef = $func;
    }

    final function validate($args) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        // call the custom function with or without the optional parameter
        $optData = $this->getOptData();
        if (isset($optData)) {
            $args_def = (strlen($this->argsDef) == 0 ?  '$optData' : $this->argsDef.', $optData');
            $f = create_function($args_def, $this->funcDef);
            $flag = $f($args, $optData);
        } else {
            $f = create_function($this->argsDef, $this->funcDef);
            $flag = $f($args);
        }

        if (!$flag) {
            $res->setValid(0);
            $res->addErrorMsg("Value [$args] is not valid");
        }
        return $res;
    }
}

class TCA_SyntaxValidator {

    public static function isValidEmail($email) {
        if (preg_match('/(@.*@)|(\.\.)|(@\.)|(\.@)|(^\.)/', $email) ||
                !preg_match('/^\S+\@(\[?)[a-zA-Z0-9\-\.]+\.([a-zA-Z]{2,3}|[0-9]{1,3})(\]?)$/', $email)
        ) {
            return false;
        }
        return true;
    }

    public static function isValidPhone($phone) {
        if (!preg_match('/^\+?[\d\s\-\.\(\)]+$/', $phone)) {
            return false;
        }
        return true;
    }

    public static function isValidPassword($password) {
        if ((!trim($password)) || (strlen($password) < 6) || (strlen($password) > 20)) {
            return false;
        }
        if (!preg_match('/^[a-z0-9\[\]\(\)!@\$\^,\.~\|=\-\+_\x7F\#\{\}]+$/i', $password)) {
            return false;
        }
        return true;
    }

    public static function isValidUsername($username) {
        if ((!trim($username)) || (strlen($username) > 20)) {
            return false;
        }
        if (!preg_match('/^[a-z0-9]+$/i', $username)) {
            return false;
        }
        return true;
    }

    public static function isValidHostname($hostname) {
        $host = strtolower($hostname);

        if (strlen($host) > 255) {
            return false;
        }
        if (!preg_match('/(?:[a-z0-9](?:[-a-z0-9]*[a-z0-9])*\.{0,1})+/', $host)) {
            return false;
        }
        return true;
    }

    public static function isValidIpAddress($ip) {
        if ((strlen($ip) > 15) || !preg_match('/^\d+\.\d+\.\d+\.\d+$/', $ip)) {
            return false;
        }
        foreach (explode(".", $ip) as $sec) {
            if ($sec > 255) {
                return false;
            }
        }
        return true;
    }
}

class TCA_BlogStructValidationRule extends TCA_ValidationRule {
    static function createRule($required) {
        $rule = new TCA_BlogStructValidationRule();
        $rule->setRequired($required);
        return $rule;
    }

    final function validate($data) {
        $res = new TCA_ValidationResult();
        $res->setValid(1);

        // Lazyman's ifelse block.
        // This switch checks to see if all required fields are present
        switch (true) {
        case (!is_array($data)):
            $res->setValid(0);
            $res->addErrorMsg(
                    "Value [$data] is not a valid struct. ".
                    "Blog Struct has to be an array."
            );
            break;
        case (!$data['name']):
            $res->setValid(0);
            $res->addErrorMsg(
                    "Passed value is not a valid struct. ".
                    "Blog struct requires 'name' field."
            );
            break;
        case (!$data['hostname']):
            $res->setValid(0);
            $res->addErrorMsg(
                    "Passed value is not a valid struct. ".
                    "Blog struct requires 'hostname' field."
            );
            break;
        case (!$data['package']):
            $res->setValid(0);
            $res->addErrorMsg(
                    "Passed value is not a valid struct. ".
                    "Blog struct requires 'package' field."
            );
            break;
        case (!isset($data['trial_mode'])):
            $res->setValid(0);
            $res->addErrorMsg(
                    "Passed value is not a valid struct. ".
                    "Blog struct requires 'trial_mode' field."
            );
            break;
        case (!$data['user']):
            $res->setValid(0);
            $res->addErrorMsg(
                    "Passed value is not a valid struct. ".
                    "Blog struct requires 'user' struct."
            );
            break;
        // Now lets check that user struct for required fields
        case (!$data['user']['username']):
            $res->setValid(0);
            $res->addErrorMsg(
                    "Passed value is not a valid struct. ".
                    "Blog_struct.user_struct requires 'username' field."
            );
            break;
        case (!$data['user']['password']):
            $res->setValid(0);
            $res->addErrorMsg(
                    "Passed value is not a valid struct. ".
                    "Blog_struct.user_struct requires 'password' field."
            );
            break;
        }
        return $res;
    }
}

?>
