<?php

interface TCA_PayloadTranslator {
    function encode($msg, $authinfo = NULL);
    function decode($msg);
}

class TCA_OPS implements TCA_PayloadTranslator {

     private $_OPS_VERSION    = '0.9';
     private $_OPT            = '';
     private $_SPACER	      = ' ';        /* indent character */
     private $_CRLF           = "\n";
     private $_MSGTYPE_STD    = 'standard';
     private $_SESSID;
     private $_MSGCNT;

     private $CRLF            = "\r\n";

     private $_log            = array();

     private $_data;


    /**
     * Class constructor
     *
     * Initialize iables, logs, etc.
     *
     * @param    array    allows for setting ious options (right now, just whether
     *                    to use compression or not on the generated XML)
     */

    function __construct($args=false) {
    	
        if (is_array($args)) {
            if ($args['option']=='compress') {
                $this->_OPT    = 'compress';
                $this->_SPACER    = '';
                $this->_CRLF    = '';
            }
        }

        $this->_SESSID = getmypid();
        $this->_MSGCNT = 0;

        $this->_log('raw','i','OPS Raw Log:');
        $this->_log('raw','i','Initialized '.date('r') );

        $this->_log('xml','i','OPS XML Log:');
        $this->_log('xml','i','Initialized '.date('r') );

    }



#
#    DECODING METHODS
#    Converts XML OPS messages into PHP data
#


    /**
     * Accepts an OPS protocol message or an file handle
     * and decodes the data into a PHP array
     *
     * @param    string         OPS message
     *
     * @return    mixed        PHP array, or error
     *
     */

    function decode($in) {

        $opsMsg = '';

        /* determine if we were passed a string or file handle */

        if (is_resource($in)) {
            # read the file into a string, then process as usual
                while (!feof($in)) {
                    $opsMsg .= fgets($in, 400);
                }
        } else {
            $opsMsg = $in;
        }


        /* log it first */

        $this->_log('xml', 'r', $opsMsg);


        /* decode and return */

        return $this->XML2PHP($opsMsg);

    }

#
#    ENCODING METHODS
#    Converts PHP data into XML OPS messages
#


    /**
     * Converts a PHP array into an OPS message
     *
     * @param    array        PHP array
     *
     * @return     string        OPS XML message
     *
     */

    function encode($array, $authinfo = NULL) {

        $this->_MSGCNT++;
        $msgId = $this->_SESSID + $this->_MSGCNT;            /* addition removes the leading zero */
        $msgType = $this->_MSGTYPE_STD;


        if ($array['protocol']) {
            $array['protocol'] = strtoupper($array['protocol']);
        }
        if ($array['action']) {
            $array['action'] = strtoupper($array['action']);
        }
        if ($array['object']) {
            $array['object'] = strtoupper($array['object']);
        }

        $xmlDataBlock = $this->PHP2XML($array);

        $opsMsg = '<?xml version="1.0" encoding="UTF-8" standalone="no" ?>' . $this->_CRLF .
            '<!DOCTYPE OPS_envelope SYSTEM "ops.dtd">' . $this->_CRLF .
            '<OPS_envelope>' . $this->_CRLF .
            $this->_SPACER . '<header>' . $this->_CRLF .
            $this->_SPACER . $this->_SPACER . '<version>' . $this->_OPS_VERSION . '</version>' . $this->_CRLF .
            $this->_SPACER . $this->_SPACER . '<msg_id>' . $msgId . '</msg_id>' . $this->_CRLF .
            $this->_SPACER . $this->_SPACER . '<msg_type>' . $msgType . '</msg_type>' . $this->_CRLF .
            $this->_SPACER . '</header>' . $this->_CRLF .
            $this->_SPACER . '<body>' . $this->_CRLF .
            $xmlDataBlock . $this->_CRLF .
            $this->_SPACER . '</body>' . $this->_CRLF .
            '</OPS_envelope>';

        # log it

        $this->_log('xml', 'w', $opsMsg);

        return $opsMsg;
    }

    /**
     * Internal method to generate error codes hashes
     *
     * @param    int         error code
     *
     * @param    string        error message
     *
     * @return    array         error hash
     *
     */

    private function _opsError($errCode,$errText) {
        return array(
            'response_code'    => $errCode,
            'response_text'    => $errText,
            'is_success'    => 0
        );
    }



    /**
     * XML Parser that converts an OPS protocol message into a PHP array
     *
     * @param    string         OPS message
     *
     * @return    mixed        PHP array, or error
     *
     */

    private function XML2PHP($msg) {

        $this->_data = NULL;

        $xp = xml_parser_create();
        xml_parser_set_option($xp, XML_OPTION_CASE_FOLDING, false);
        xml_parser_set_option($xp, XML_OPTION_SKIP_WHITE, true);
        xml_parser_set_option($xp, XML_OPTION_TARGET_ENCODING, 'ISO-8859-1');

        if (!xml_parse_into_struct($xp,$msg,$vals,$index)) {
            $error = sprintf('XML error: %s at line %d',
                xml_error_string(xml_get_error_code($xp)),
                xml_get_current_line_number($xp)
            );
            xml_parser_free($xp);
            throw new TCA_XMLParseException($error);
        }

        xml_parser_free($xp);

        $temp = $depth = array();

        foreach($vals as $value) {

            switch ($value['tag']) {
              case 'OPS_envelope':
              case 'header':
              case 'body':
              case 'data_block':
                break;

              case 'version':
              case 'msg_id':
              case 'msg_type':
                $key = '_OPS_' . $value['tag'];
                $temp[$key] = $value['value'];
                break;

              case 'item':
                $key = $value['attributes']['key'];

                switch ($value['type']) {
                  case 'open':
                    array_push($depth, $key);
                    break;

                  case 'complete':
                    array_push($depth, $key);
                    $p = join('::',$depth);
                    $temp[$p] = $value['value'];
                    array_pop($depth);
                    break;

                  case 'close':
                    array_pop($depth);
                    break;

                }

                break;

              case 'dt_assoc':
              case 'dt_array':
                break;

            }
        }

        foreach ($temp as $key=>$value) {

            $levels = explode('::',$key);
            $numLevels = count($levels);

            if ($numLevels==1) {
                $this->_data[$levels[0]] = $value;
            } else {
                $pointer = &$this->_data;
                for ($i=0; $i<$numLevels; $i++) {
                    if ( !isset( $pointer[$levels[$i]] ) ) {
                        $pointer[$levels[$i]] = array();
                    }
                    $pointer = &$pointer[$levels[$i]];
                }
                $pointer = $value;
            }

        }

        return ($this->_data);
    }



    /**
     * Converts a PHP array into an OPS data_block tag
     *
     * @param    array        PHP array
     *
     * @return     string        OPS data_block tag
     *
     */

    private function PHP2XML($data) {
        return str_repeat($this->_SPACER,2) . '<data_block>' .
            $this->_convertData($data, 3) .
            $this->_CRLF . str_repeat($this->_SPACER,2) . '</data_block>';
    }


    /**
     * Recursivly converts PHP data into XML
     *
     * @param    mixed        PHP array or data
     *
     * @param    int            ident level
     *
     * @return     string        XML string
     *
     */

    private function _convertData(&$array, $indent=0) {
        $string = '';
        $IND = str_repeat($this->_SPACER,$indent);

        if (is_array($array)) {

            if ($this->_is_assoc($array)) {        # HASH REFERENCE
                $string .= $this->_CRLF . $IND . '<dt_assoc>';
                $end = '</dt_assoc>';
            } else {                # ARRAY REFERENCE
                $string .= $this->_CRLF . $IND . '<dt_array>';
                $end = '</dt_array>';
            }

            foreach ($array as $k=>$v) {

                $indent++;

                /* don't encode some types of stuff */
                if ((gettype($v)=='resource') || (gettype($v)=='user function') || (gettype($v)=='unknown type')) {
                    continue;
                }

                $string .= $this->_CRLF . $IND . '<item key="' . $k . '"';

                if (gettype($v)=='object' && get_class($v)) {
                    $string .= ' class="' . get_class($v) . '"';
                }

                $string .= '>';

                if (is_array($v) || is_object($v)) {
                    $string .= $this->_convertData($v, $indent+1);
                    $string .= $this->_CRLF . $IND . '</item>';
                } else {
                    $string .= $this->_quoteXMLChars($v) . '</item>';
                }

                $indent--;
            }

            $string .= $this->_CRLF . $IND . $end;

        } else {                    # SCALAR

            $string .= $this->_CRLF . $IND . '<dt_scalar>' .
                $this->_quoteXMLChars($array) . '</dt_scalar>';
        }

        return $string;

    }


    /**
     * Quotes special XML characters
     *
     * @param    string        string to quote
     *
     * @return     string        quoted string
     *
     */

    private function _quoteXMLChars($string) {
        $search  = array ('&', '<', '>', "'", '"');
        $replace = array ('&amp;', '&lt;', '&gt;', '&apos;', '&quot;');
        $string = str_replace($search, $replace, $string);
        $string = utf8_encode($string);
        return $string;
    }


    /**
     * Determines if an array is associative or not, since PHP
     * doesn't really distinguish between the two, but Perl/OPS does
     *
     * @param    array        array to check
     *
     * @return     boolean        true if the array is associative
     *
     */

    private function _is_assoc(&$array) {
        if (is_array($array)) {
            foreach ($array as $k=>$v) {
            	if (!is_int($k)) {
                	return true;
                }
            }
        }
        return false;
    }


    /**
     * Internal loggging method
     *
     * @param    string         which log to log to
     *
     * @param    string        type of log message ('r'ead, 'w'rite, 'i'nfo or 'e'rror)
     *
     * @param    int            message
     *
     */

    private function _log($log, $type, $msg) {
        $types = array(
            'r'    => 'read',
            'w'    => 'write',
            'e'    => 'error',
            'i' => 'info'
        );

        if ($log=='xml') {
            $this->log[$log][] = sprintf("[% 6s:%06d] %s\n",
                strtoupper($types[$type]),
                ($type=='e' || $type=='i') ? 0 : strlen($msg),
                $msg
            );
        } else {
            $this->log[$log][] = sprintf("[% 6s:%06d] %s\n",
                strtoupper($types[$type]),
                ($type=='e' || $type=='i') ? 0 : strlen($msg),
                ($type=='e' || $type=='i') ? $msg : bin2hex($msg)
            );
        }
    }


    /**
     * Show internal log
     *
     * @param    string         which log to log show, 'raw' or 'xml'
     *
     * @param    string        format to display: 'html' (default) or 'raw'
     *
     */

    function showLog($log, $format='html') {
        echo '<PRE>';
        foreach ($this->log[$log] as $line) {
            switch ($format) {
              case 'raw':
                echo $line . "\n";
                break;
              case 'html':
              default:
                echo htmlEntities($line) . "\n";
                break;
            }
        }
        echo '</PRE>';
    }
}

class TCA_XMLRPCBlogware implements TCA_PayloadTranslator {
     private $_SPACER	      = ' ';        /* indent character */
     private $_CRLF           = "\n";

    function encode($array, $authinfo = NULL) {
    	$params = array();
    	
    	// version
    	array_push($params,'1.0');
    	// username
    	array_push($params,$authinfo->getUsername());
    	// password
    	array_push($params,$authinfo->getPassword());
   
	// This is a temp fix that removes names and keeps the order of the input.
	// This is done b/c the blogware XMLRPC structure doesn't support Value/Name pairs.
    	if($array['attributes']) {
		foreach($array['attributes'] as $attr) {
			array_push($params,$attr);
		}
	}
    	
    	/*
	// Create name/value pairs for shipment
    	$param = array('version'=>'1.0');
    	array_push($params,$param);
    	$param = array('username'=>$authinfo->getUsername());
    	array_push($params,$param);
    	$param = array('password'=>$authinfo->getPassword());
    	array_push($params,$param);
    	    	
    	foreach($array['attributes'] as $key=>$attr) {
    		$param = array($key=>$attr);
    		array_push($params,$param);
    	}
    	
    	*/
    	
        $xmlMsg = xmlrpc_encode_request($array['object'].'.'.$array['action'],$params); // Use built-in XMLRPC function.

        return $xmlMsg;
    }
    
    function decode($in) {
	$response = array();
	$rawResponse = xmlrpc_decode($in); // use built-in XMLRPC parser.

	// Blogware XMLRPC commonly returns a bool for succes or failure
	if(is_bool($rawResponse)) {
		$response['is_success'] = 1;
		$response['response_code'] = 1;
		$response['response_text'] = "Command Successfully Executed";
		$response['attributes'] = array();
		$response['attributes']['response'] = $rawResponse?'1':'0';
	} else { // Blogware XMLRPC passed a struct!
		if($rawResponse['faultCode']) {
			$response['is_success'] = 0;
			$response['response_code'] = $rawResponse['faultCode'];
			$response['response_text'] = $rawResponse['faultString'];
		} else {
			$response['is_success'] = 1;
			$response['response_code'] = 1;
			$response['response_text'] = "Command Successfully Executed";

			if(is_array($rawResponse)) {
				$response['attributes'] = $rawResponse;
			} else {
				$response['attributes'] = array();
				$response['attributes']['response'] = $rawResponse;
			}
		}
	}
	
	return $response;
    }
}

?>
